@@ -310,8 +310,8 @@ BOOLEAN iiAllStart(procinfov pi, char *p,feBufferTypes t, int l)
                pi, l );
   #ifndef LIBSINGULAR
   // see below:
-  int save1=(test & ~TEST_RINGDEP_OPTS);
-  int save2=verbose;
+  BITSET save1=(test & ~TEST_RINGDEP_OPTS);
+  BITSET save2=verbose;
   #endif
   BOOLEAN err=yyparse();
   if (sLastPrinted.rtyp!=0)
@@ -321,7 +321,7 @@ BOOLEAN iiAllStart(procinfov pi, char *p,feBufferTypes t, int l)
   #ifndef LIBSINGULAR
   // the access to optionStruct and verboseStruct do not work
   // on x86_64-Linux for pic-code
-  int save11= ( test & ~TEST_RINGDEP_OPTS);
+  BITSET save11= ( test & ~TEST_RINGDEP_OPTS);
   if ((TEST_V_ALLWARN) &&
   (t==BT_proc) &&
   ((save1!=save11)||(save2!=verbose)) &&
@@ -436,7 +436,7 @@ BOOLEAN iiPStart(idhdl pn, sleftv  * v)
 #ifdef USE_IILOCALRING
 ring    *iiLocalRing;
 #endif
-sleftv  *iiRETURNEXPR;
+sleftv  iiRETURNEXPR;
 int     iiRETURNEXPR_len=0;
 
 #ifdef RDEBUG
@@ -473,11 +473,6 @@ static void iiCheckNest()
 {
   if (myynest >= iiRETURNEXPR_len-1)
   {
-    iiRETURNEXPR=(sleftv *)omreallocSize(iiRETURNEXPR,
-                                   iiRETURNEXPR_len*sizeof(sleftv),
-                                   (iiRETURNEXPR_len+16)*sizeof(sleftv));
-    omMarkAsStaticAddr(iiRETURNEXPR);
-    memset(&(iiRETURNEXPR[iiRETURNEXPR_len]),0,16*sizeof(sleftv));
 #ifdef USE_IILOCALRING
     iiLocalRing=(ring *)omreallocSize(iiLocalRing,
                                    iiRETURNEXPR_len*sizeof(ring),
@@ -487,7 +482,7 @@ static void iiCheckNest()
     iiRETURNEXPR_len+=16;
   }
 }
-sleftv * iiMake_proc(idhdl pn, package pack, sleftv* sl)
+BOOLEAN iiMake_proc(idhdl pn, package pack, sleftv* sl)
 {
   int err;
   procinfov pi = IDPROC(pn);
@@ -495,14 +490,13 @@ sleftv * iiMake_proc(idhdl pn, package pack, sleftv* sl)
   {
     Werror("'%s::%s()' is a local procedure and cannot be accessed by an user.",
            pi->libname, pi->procname);
-    return NULL;
+    return TRUE;
   }
   iiCheckNest();
 #ifdef USE_IILOCALRING
   iiLocalRing[myynest]=currRing;
   //Print("currRing(%d):%s(%x) in %s\n",myynest,IDID(currRingHdl),currRing,IDID(pn));
 #endif
-  iiRETURNEXPR[myynest+1].Init();
   procstack->push(pi->procname);
   if ((traceit&TRACE_SHOW_PROC)
   || (pi->trace_flag&TRACE_SHOW_PROC))
@@ -541,7 +535,7 @@ sleftv * iiMake_proc(idhdl pn, package pack, sleftv* sl)
     case LANG_C:
                  leftv res = (leftv)omAlloc0Bin(sleftv_bin);
                  err = (pi->data.o.function)(res, sl);
-                 iiRETURNEXPR[myynest+1].Copy(res);
+                 iiRETURNEXPR.Copy(res);
                  omFreeBin((ADDRESS)res,  sleftv_bin);
                  break;
   }
@@ -559,8 +553,8 @@ sleftv * iiMake_proc(idhdl pn, package pack, sleftv* sl)
 #endif
   if (err)
   {
-    iiRETURNEXPR[myynest+1].CleanUp();
-    //iiRETURNEXPR[myynest+1].Init(); //done by CleanUp
+    iiRETURNEXPR.CleanUp();
+    //iiRETURNEXPR.Init(); //done by CleanUp
   }
 #ifdef USE_IILOCALRING
 #if 0
@@ -570,10 +564,10 @@ sleftv * iiMake_proc(idhdl pn, package pack, sleftv* sl)
   {
     if (currRing!=NULL)
     {
-      if (((iiRETURNEXPR[myynest+1].Typ()>BEGIN_RING)
-        && (iiRETURNEXPR[myynest+1].Typ()<END_RING))
-      || ((iiRETURNEXPR[myynest+1].Typ()==LIST_CMD)
-        && (lRingDependend((lists)iiRETURNEXPR[myynest+1].Data()))))
+      if (((iiRETURNEXPR.Typ()>BEGIN_RING)
+        && (iiRETURNEXPR.Typ()<END_RING))
+      || ((iiRETURNEXPR.Typ()==LIST_CMD)
+        && (lRingDependend((lists)iiRETURNEXPR.Data()))))
       {
         //idhdl hn;
         const char *n;
@@ -588,7 +582,7 @@ sleftv * iiMake_proc(idhdl pn, package pack, sleftv* sl)
         if (nh!=NULL)          n=nh->id;
         else                   n="none";
         Werror("ring change during procedure call: %s -> %s (level %d)",o,n,myynest);
-        iiRETURNEXPR[myynest+1].CleanUp();
+        iiRETURNEXPR.CleanUp();
         err=TRUE;
       }
     }
@@ -613,10 +607,10 @@ sleftv * iiMake_proc(idhdl pn, package pack, sleftv* sl)
     //if (currRingHdl!=NULL)
     //Print(" curr:%s\n",IDID(currRingHdl));
     //Print("pr:%x, curr: %x\n",procstack->cRing,currRing);
-    if (((iiRETURNEXPR[myynest+1].Typ()>BEGIN_RING)
-      && (iiRETURNEXPR[myynest+1].Typ()<END_RING))
-    || ((iiRETURNEXPR[myynest+1].Typ()==LIST_CMD)
-      && (lRingDependend((lists)iiRETURNEXPR[myynest+1].Data()))))
+    if (((iiRETURNEXPR.Typ()>BEGIN_RING)
+      && (iiRETURNEXPR.Typ()<END_RING))
+    || ((iiRETURNEXPR.Typ()==LIST_CMD)
+      && (lRingDependend((lists)iiRETURNEXPR.Data()))))
     {
       //idhdl hn;
       char *n;
@@ -641,7 +635,7 @@ sleftv * iiMake_proc(idhdl pn, package pack, sleftv* sl)
       if (currRing==NULL)
       {
         Werror("ring change during procedure call: %s -> %s",o,n);
-        iiRETURNEXPR[myynest+1].CleanUp();
+        iiRETURNEXPR.CleanUp();
         err=TRUE;
       }
     }
@@ -662,8 +656,8 @@ sleftv * iiMake_proc(idhdl pn, package pack, sleftv* sl)
   }
   procstack->pop();
   if (err)
-    return NULL;
-  return &iiRETURNEXPR[myynest+1];
+    return TRUE;
+  return FALSE;
 }
 
 /*2
@@ -686,7 +680,6 @@ BOOLEAN iiEStart(char* example, procinfo *pi)
     printf("entering example (level %d)\n",myynest);
   }
   myynest++;
-  iiRETURNEXPR[myynest].Init();
 
   err=iiAllStart(pi,example,BT_example,(pi != NULL ? pi->data.s.example_lineno: 0));
 
@@ -734,6 +727,25 @@ BOOLEAN iiEStart(char* example, procinfo *pi)
   return err;
 }
 
+int huhu_mod_init(SModulFunctions*){ return 0; }
+
+#define SI_GET_BUILTIN_MOD_INIT(name) \
+  int name##_mod_init(SModulFunctions*); \
+  if (strcmp(libname, #name ".so") == 0) {  return name##_mod_init; }
+
+
+SModulFunc_t
+iiGetBuiltinModInit(char* libname)
+{
+  SI_FOREACH_BUILTIN(SI_GET_BUILTIN_MOD_INIT)
+
+  return NULL;
+}
+
+
+#undef SI_GET_BUILTIN_MOD_INIT
+
+
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 BOOLEAN iiTryLoadLib(leftv v, const char *id)
 {
@@ -742,11 +754,7 @@ BOOLEAN iiTryLoadLib(leftv v, const char *id)
   char *libname = (char *)omAlloc(strlen(id)+5);
   const char *suffix[] = { "", ".lib", ".so", ".sl", NULL };
   int i = 0;
-  FILE *fp;
-  package pack;
-  idhdl packhdl;
   lib_types LT;
-
   for(i=0; suffix[i] != NULL; i++)
   {
     sprintf(libname, "%s%s", id, suffix[i]);
@@ -764,7 +772,7 @@ BOOLEAN iiTryLoadLib(leftv v, const char *id)
       #endif
       else if (LT==LT_BUILTIN)
       {
-        LoadResult=load_builtin(s,FALSE,(SModulFunc_t)NULL);
+        LoadResult=load_builtin(s,FALSE, iiGetBuiltinModInit(s));
       }
       if(!LoadResult )
       {
@@ -816,14 +824,9 @@ BOOLEAN iiLocateLib(const char* lib, char* where)
 BOOLEAN iiLibCmd( char *newlib, BOOLEAN autoexport, BOOLEAN tellerror, BOOLEAN force )
 {
   char libnamebuf[128];
-  procinfov pi;
-  idhdl h;
   idhdl pl;
-  idhdl hl;
-  long pos = 0L;
   char *plib = iiConvName(newlib);
   FILE * fp = feFopen( newlib, "r", libnamebuf, tellerror );
-  int lines = 1;
   BOOLEAN LoadResult = TRUE;
 
   if (fp==NULL)
@@ -894,7 +897,6 @@ static void iiRunInit(package p)
   {
     int save=yylineno;
     myynest++;
-    procinfo *pi=(procinfo*)IDDATA(h);
     //PrintS("mod_init found\n");
     iiMake_proc(h,p,NULL);
     myynest--;
@@ -902,7 +904,7 @@ static void iiRunInit(package p)
   }
 }
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
-BOOLEAN iiLoadLIB(FILE *fp, char *libnamebuf, char*newlib,
+BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, char*newlib,
              idhdl pl, BOOLEAN autoexport, BOOLEAN tellerror)
 {
   extern FILE *yylpin;
@@ -1102,6 +1104,10 @@ BOOLEAN load_modules(char *newlib, char *fullname, BOOLEAN autoexport)
       goto load_modules_end;
     }
   }
+  if (dynl_check_opened(FullName)) {
+    if (BVERBOSE(V_LOAD_LIB)) Warn( "%s already loaded", fullname);
+    return FALSE;
+  }
   if((IDPACKAGE(pl)->handle=dynl_open(FullName))==(void *)NULL)
   {
     Werror("dynl_open failed:%s", dynl_error());
@@ -1142,20 +1148,22 @@ BOOLEAN load_builtin(char *newlib, BOOLEAN autoexport, SModulFunc_t init)
   typedef int (*fktn_t)(int(*iiAddCproc)(const char *libname, const char *procname,
                                BOOLEAN pstatic,
                                BOOLEAN(*func)(leftv res, leftv v)));
-  SModulFunc_t fktn;
   idhdl pl;
   char *plib = iiConvName(newlib);
-  BOOLEAN RET=TRUE;
-  int token;
 
   pl = IDROOT->get(plib,0);
-  if (pl==NULL)
+  if (pl!=NULL)
   {
-    pl = enterid( plib,0, PACKAGE_CMD, &IDROOT,
-                  TRUE );
-    IDPACKAGE(pl)->language = LANG_C;
-    IDPACKAGE(pl)->libname=omStrDup(newlib);
+    if (BVERBOSE(V_LOAD_LIB)) Warn( "(builtin) %s already loaded", newlib);
+    omFree(plib);
+    return FALSE;
   }
+
+  pl = enterid( plib,0, PACKAGE_CMD, &IDROOT,
+                TRUE );
+  IDPACKAGE(pl)->language = LANG_C;
+  IDPACKAGE(pl)->libname=omStrDup(newlib);
+
   IDPACKAGE(pl)->handle=(void *)NULL;
   SModulFunctions sModulFunctions;
 
@@ -1168,7 +1176,7 @@ BOOLEAN load_builtin(char *newlib, BOOLEAN autoexport, SModulFunc_t init)
     else            sModulFunctions.iiAddCproc = iiAddCproc;
     (*init)(&sModulFunctions);
   }
-  if (BVERBOSE(V_LOAD_LIB)) Print( "// ** loaded %s \n", newlib);
+  if (BVERBOSE(V_LOAD_LIB)) Print( "// ** loaded (builtin) %s \n", newlib);
   currPack->loaded=1;
   currPack=s;
 
