@@ -23,10 +23,9 @@
 #include <kernel/kutil.h>
 
 //------GLOBALS-------
-static int m_s,v_s,vectorized,VarN1,offset;
+static int offset;
 static jList *T,*Q;
 static TreeM *G;
-static Poly *phD;
 static NodeM *FreeNodes;
 static int degree_compatible;
 static int (*ListGreatMove)(jList *,jList *,poly);
@@ -183,7 +182,6 @@ void PNF(Poly *p, TreeM *F)
 void NFL(Poly *p, TreeM *F)
 {
   Poly *f;
-  int g1,f1,gg;
 
   if ((f=is_div_(F,p->lead))==NULL) return;
 
@@ -236,7 +234,7 @@ void NFL(Poly *p, TreeM *F)
   }
 
   //if (TEST_OPT_PROT) { PrintS("R"); mflush(); }
-  int old_size, count;
+  int count;
   count=0;
   while(f && p->root)
   {
@@ -293,8 +291,7 @@ void NFL(Poly *p, TreeM *F)
 
 int ValidatePoly(Poly *x, TreeM *F)
 {
-  Poly *f,*g;
-  int g1,f1;
+  Poly *g;
 
   if (x->root) return 1;
 
@@ -961,7 +958,7 @@ void Initialization(char *Ord)
   Define(&G);
 };
 
-static Poly *h,*f;
+static Poly *h;
 
 void insert_in_G(Poly *x)
 {
@@ -988,8 +985,6 @@ void Q2TG()
 
 int ComputeBasis(jList *_lT,jList *_lQ)
 {
-  int gb_l,i,ret_value=1;
-
   T=_lT; Q=_lQ;
 
 //  Debug();
