@@ -55,7 +55,7 @@ char * newstruct_String(blackbox *b, void *d)
 
     if (p!=NULL)
     {
-      leftv sl;
+      BOOLEAN sl;
       sleftv tmp;
       memset(&tmp,0,sizeof(tmp));
       tmp.rtyp=ad->id;
@@ -68,14 +68,15 @@ char * newstruct_String(blackbox *b, void *d)
       hh.data.pinf=p->p;
       sl=iiMake_proc(&hh,NULL,&tmp);
 
-      if (sl->Typ() == STRING_CMD)
+      if ((!sl)&& (iiRETURNEXPR.Typ() == STRING_CMD))
       {
-        char *res = omStrDup((char*)sl->Data());
-        sl->CleanUp();
+        char *res = omStrDup((char*)iiRETURNEXPR.CopyD());
+        iiRETURNEXPR.CleanUp();
+        iiRETURNEXPR.Init();
         return res;
       }
-      else
-        sl->CleanUp();      
+      iiRETURNEXPR.CleanUp();
+      iiRETURNEXPR.Init();
     }
 
     lists l=(lists)d;
@@ -169,12 +170,12 @@ BOOLEAN newstruct_equal(int op, leftv l, leftv r)
   assume(ll->data != NULL);
   newstruct_desc nt=(newstruct_desc)ll->data;
   newstruct_proc p=nt->procs;
-  
+
   while( (p!=NULL) && ((p->t!='=')||(p->args!=1)) ) p=p->next;
 
   if (p!=NULL)
   {
-    leftv sl;
+    BOOLEAN sl;
     idrec hh;
     memset(&hh,0,sizeof(hh));
     hh.id=Tok2Cmdname(p->t);
@@ -184,15 +185,41 @@ BOOLEAN newstruct_equal(int op, leftv l, leftv r)
     memset(&tmp,0,sizeof(sleftv));
     tmp.Copy(r);
     sl = iiMake_proc(&hh, NULL, &tmp);
-    if (sl != NULL)
+    if (!sl)
     {
-      if (sl->Typ() == op) { l->Copy(sl); return FALSE;}
-      else sl->CleanUp();
+      if (iiRETURNEXPR.Typ() == op)
+      {
+        l->Copy(&iiRETURNEXPR);
+        iiRETURNEXPR.Init();
+        return FALSE;
+      }
+      iiRETURNEXPR.CleanUp();
+      iiRETURNEXPR.Init();
     }
   }
   return TRUE;
 }
 
+void lClean_newstruct(lists l)
+{
+  if (l->nr>=0)
+  {
+    int i;
+    ring r=NULL;
+    for(i=l->nr;i>=0;i--)
+    {
+      if ((i>0) && (l->m[i-1].rtyp==RING_CMD))
+        r=(ring)(l->m[i-1].data);
+      else
+        r=NULL;
+      l->m[i].CleanUp(r);
+    }
+    omFreeSize((ADDRESS)l->m, (l->nr+1)*sizeof(sleftv));
+    l->nr=-1;
+  }
+  omFreeBin((ADDRESS)l,slists_bin);
+}
+
 BOOLEAN newstruct_Assign(leftv l, leftv r)
 {
   if (r->Typ()>MAX_TOK)
@@ -234,7 +261,7 @@ BOOLEAN newstruct_Assign(leftv l, leftv r)
       if (l->Data()!=NULL)
       {
         lists n1=(lists)l->Data();
-        n1->Clean(); n1=NULL;
+        lClean_newstruct(n1);
       }
       lists n2=(lists)r->Data();
       n2=lCopy_newstruct(n2);
@@ -272,7 +299,7 @@ BOOLEAN newstruct_Op1(int op, leftv res, leftv arg)
 
   if (p!=NULL)
   {
-    leftv sl;
+    BOOLEAN sl;
     sleftv tmp;
     memset(&tmp,0,sizeof(sleftv));
     tmp.Copy(arg);
@@ -282,10 +309,11 @@ BOOLEAN newstruct_Op1(int op, leftv res, leftv arg)
     hh.typ=PROC_CMD;
     hh.data.pinf=p->p;
     sl=iiMake_proc(&hh,NULL,&tmp);
-    if (sl==NULL) return TRUE;
+    if (sl) return TRUE;
     else
     {
-      res->Copy(sl);
+      res->Copy(&iiRETURNEXPR);
+      iiRETURNEXPR.Init();
       return FALSE;
     }
   }
@@ -400,7 +428,7 @@ BOOLEAN newstruct_Op2(int op, leftv res, leftv a1, leftv a2)
   while((p!=NULL) && ( (p->t!=op) || (p->args!=2) )) p=p->next;
   if (p!=NULL)
   {
-    leftv sl;
+    BOOLEAN sl;
     sleftv tmp;
     memset(&tmp,0,sizeof(sleftv));
     tmp.Copy(a1);
@@ -412,10 +440,11 @@ BOOLEAN newstruct_Op2(int op, leftv res, leftv a1, leftv a2)
     hh.typ=PROC_CMD;
     hh.data.pinf=p->p;
     sl=iiMake_proc(&hh,NULL,&tmp);
-    if (sl==NULL) return TRUE;
+    if (sl) return TRUE;
     else
     {
-      res->Copy(sl);
+      res->Copy(&iiRETURNEXPR);
+      iiRETURNEXPR.Init();
       return FALSE;
     }
   }
@@ -445,7 +474,7 @@ BOOLEAN newstruct_OpM(int op, leftv res, leftv args)
 
   if (p!=NULL)
   {
-    leftv sl;
+    BOOLEAN sl;
     sleftv tmp;
     memset(&tmp,0,sizeof(sleftv));
     tmp.Copy(args);
@@ -455,36 +484,17 @@ BOOLEAN newstruct_OpM(int op, leftv res, leftv args)
     hh.typ=PROC_CMD;
     hh.data.pinf=p->p;
     sl=iiMake_proc(&hh,NULL,&tmp);
-    if (sl==NULL) return TRUE;
+    if (sl) return TRUE;
     else
     {
-      res->Copy(sl);
+      res->Copy(&iiRETURNEXPR);
+      iiRETURNEXPR.Init();
       return FALSE;
     }
   }
   return blackboxDefaultOpM(op,res,args);
 }
 
-void lClean_newstruct(lists l)
-{
-  if (l->nr>=0)
-  {
-    int i;
-    ring r=NULL;
-    for(i=l->nr;i>=0;i--)
-    {
-      if ((i>0) && (l->m[i-1].rtyp==RING_CMD))
-        r=(ring)(l->m[i-1].data);
-      else
-        r=NULL;
-      l->m[i].CleanUp(r);
-    }
-    omFreeSize((ADDRESS)l->m, (l->nr+1)*sizeof(sleftv));
-    l->nr=-1;
-  }
-  omFreeBin((ADDRESS)l,slists_bin);
-}
-
 void newstruct_destroy(blackbox *b, void *d)
 {
   if (d!=NULL)
@@ -582,7 +592,7 @@ void newstruct_Print(blackbox *b,void *d)
     p=p->next;
   if (p!=NULL)
   {
-    leftv sl;
+    BOOLEAN sl;
     sleftv tmp;
     memset(&tmp,0,sizeof(tmp));
     tmp.rtyp=dd->id;
@@ -593,6 +603,8 @@ void newstruct_Print(blackbox *b,void *d)
     hh.typ=PROC_CMD;
     hh.data.pinf=p->p;
     sl=iiMake_proc(&hh,NULL,&tmp);
+    if (!sl) iiRETURNEXPR.CleanUp();
+    iiRETURNEXPR.Init();
   }
   else
     blackbox_default_Print(b,d);
