@@ -215,6 +215,23 @@ BOOLEAN slOpen(si_link l, short flag, leftv h)
   return res;
 }
 
+BOOLEAN slPrepClose(si_link l)
+{
+
+  if(! SI_LINK_OPEN_P(l))
+    return FALSE;
+
+  BOOLEAN res = TRUE;
+  if (l->m->PrepClose != NULL)
+  {
+    res = l->m->PrepClose(l);
+    if (res)
+      Werror("close: Error for link of type: %s, mode: %s, name: %s",
+           l->m->type, l->mode, l->name);
+  }
+  return res;
+}
+
 BOOLEAN slClose(si_link l)
 {
 
@@ -234,7 +251,6 @@ BOOLEAN slClose(si_link l)
 
 leftv slRead(si_link l, leftv a)
 {
-  char *mode;
   leftv v = NULL;
   if( ! SI_LINK_R_OPEN_P(l)) // open r ?
   {
@@ -335,6 +351,7 @@ BOOLEAN slDump(si_link l)
     if (res)
       Werror("dump: Error for link of type %s, mode: %s, name: %s",
              l->m->type, l->mode, l->name);
+    if (!SI_LINK_R_OPEN_P(l)) slClose(l); // do not close r/w links
     return res;
   }
   else
@@ -661,7 +678,6 @@ static const char* GetIdString(idhdl h)
       {
         lists l = IDLIST(h);
         int i, nl = l->nr + 1;
-        char *name;
 
         for (i=0; i<nl; i++)
           if (GetIdString((idhdl) &(l->m[i])) == NULL) return NULL;
@@ -736,7 +752,7 @@ static int DumpRhs(FILE *fd, idhdl h)
   }
   else  if (type_id == STRING_CMD)
   {
-    char *pstr = IDSTRING(h), c;
+    char *pstr = IDSTRING(h);
     fputc('"', fd);
     while (*pstr != '\0')
     {
@@ -749,17 +765,20 @@ static int DumpRhs(FILE *fd, idhdl h)
   else  if (type_id == PROC_CMD)
   {
     procinfov pi = IDPROC(h);
-    if (pi->language == LANG_SINGULAR) {
+    if (pi->language == LANG_SINGULAR)
+    {
       if( pi->data.s.body==NULL) iiGetLibProcBuffer(pi);
-      char *pstr = pi->data.s.body, c;
+      char *pstr = pi->data.s.body;
       fputc('"', fd);
-      while (*pstr != '\0') {
+      while (*pstr != '\0')
+      {
         if (*pstr == '"' || *pstr == '\\') fputc('\\', fd);
         fputc(*pstr, fd);
         pstr++;
       }
       fputc('"', fd);
-    } else fputs("(null)", fd);
+    }
+    else fputs("(null)", fd);
   }
   else
   {
@@ -767,7 +786,10 @@ static int DumpRhs(FILE *fd, idhdl h)
 
     if (rhs == NULL) return EOF;
 
-    if (type_id == INTVEC_CMD) fprintf(fd, "intvec(");
+    BOOLEAN need_klammer=FALSE;
+    if (type_id == INTVEC_CMD) { fprintf(fd, "intvec(");need_klammer=TRUE; }
+    else if (type_id == IDEAL_CMD) { fprintf(fd, "ideal(");need_klammer=TRUE; }
+    else if (type_id == MODUL_CMD) { fprintf(fd, "module(");need_klammer=TRUE; }
 
     if (fprintf(fd, "%s", rhs) == EOF) return EOF;
     omFree(rhs);
@@ -780,7 +802,7 @@ static int DumpRhs(FILE *fd, idhdl h)
       rhs = StringAppendS("");
       if (fprintf(fd, "; minpoly = %s", rhs) == EOF) return EOF;
     }
-    else if (type_id == INTVEC_CMD) fprintf(fd, ")");
+    else if (need_klammer) fprintf(fd, ")");
   }
   return 1;
 }
@@ -829,7 +851,7 @@ static si_link_extension slTypeInit(si_link_extension s, const char* type)
   s->next = NULL;
   si_link_extension ns = (si_link_extension)omAlloc0Bin(s_si_link_extension_bin);
 
-  if (0) 0;
+  if (0) 0; // dummy
 #ifdef HAVE_MPSR
   else if (strcmp(type, "MPfile") == 0)
     s->next = slInitMPFileExtension(ns);
@@ -871,7 +893,7 @@ void slStandardInit()
   si_link_root=(si_link_extension)omAlloc0Bin(s_si_link_extension_bin);
   si_link_root->Open=slOpenAscii;
   si_link_root->Close=slCloseAscii;
-  si_link_root->Kill=slCloseAscii;
+  si_link_root->Kill=NULL;
   si_link_root->Read=slReadAscii;
   si_link_root->Read2=slReadAscii2;
   si_link_root->Write=slWriteAscii;
