@@ -27,6 +27,7 @@
 
 #include <Singular/tok.h>
 #include <Singular/ipid.h>
+#include <Singular/ipshell.h>
 #include <omalloc/omalloc.h>
 #include <kernel/ring.h>
 #include <kernel/matpol.h>
@@ -35,6 +36,7 @@
 #include <kernel/longrat.h>
 #include <kernel/ideals.h>
 #include <kernel/intvec.h>
+#include <kernel/bigintmat.h>
 #include <kernel/options.h>
 #include <kernel/timer.h>
 #include <Singular/subexpr.h>
@@ -56,7 +58,7 @@
 //#define HAVE_PSELECT
 //#endif
 
-#define SSI_VERSION 3
+#define SSI_VERSION 5
 
 typedef struct
 {
@@ -67,6 +69,7 @@ typedef struct
   int fd_read,fd_write; /* only valid for fork/tcp mode*/
   char level;
   char send_quit_at_exit;
+  char quit_sent;
 
 } ssiInfo;
 
@@ -303,7 +306,12 @@ void ssiWriteCommand(si_link l, command D)
 
 void ssiWriteProc(ssiInfo *d,procinfov p)
 {
-  ssiWriteString(d,p->data.s.body);
+  if (p->data.s.body==NULL)
+    iiGetLibProcBuffer(p);
+  if (p->data.s.body!=NULL)
+    ssiWriteString(d,p->data.s.body);
+  else
+    ssiWriteString(d,"");
 }
 
 void ssiWriteList(si_link l,lists dd)
@@ -330,6 +338,29 @@ void ssiWriteIntvec(ssiInfo *d,intvec * v)
   }
   SSI_UNBLOCK_CHLD;
 }
+void ssiWriteIntmat(ssiInfo *d,intvec * v)
+{
+  SSI_BLOCK_CHLD;
+  fprintf(d->f_write,"%d %d ",v->rows(),v->cols());
+  int i;
+  for(i=0;i<v->length();i++)
+  {
+    fprintf(d->f_write,"%d ",(*v)[i]);
+  }
+  SSI_UNBLOCK_CHLD;
+}
+
+void ssiWriteBigintmat(ssiInfo *d,bigintmat * v)
+{
+  SSI_BLOCK_CHLD;
+  fprintf(d->f_write,"%d %d ",v->rows(),v->cols());
+  int i;
+  for(i=0;i<v->length();i++)
+  {
+    ssiWriteBigInt(d,(*v)[i]);
+  }
+  SSI_UNBLOCK_CHLD;
+}
 
 char *ssiReadString(ssiInfo *d)
 {
@@ -374,14 +405,12 @@ number ssiReadBigInt(ssiInfo *d)
    }
 }
 
-number ssiReadNumber(ssiInfo *d)
+static number ssiReadQNumber(ssiInfo *d)
 {
-  if (rField_is_Q(d->r))
+  int sub_type=-1;
+  sub_type=s_readint(d->f_read);
+  switch(sub_type)
   {
-     int sub_type=-1;
-     sub_type=s_readint(d->f_read);
-     switch(sub_type)
-     {
      case 0:
      case 1:
        {// read mpz_t, mpz_t
@@ -426,7 +455,14 @@ number ssiReadNumber(ssiInfo *d)
 
      default: Werror("error in reading number: invalid subtype %d",sub_type);
               return NULL;
-     }
+  }
+  return NULL;
+}
+number ssiReadNumber(ssiInfo *d)
+{
+  if (rField_is_Q(d->r))
+  {
+     return ssiReadQNumber(d);
   }
   else if (rField_is_Zp(d->r))
   {
@@ -639,6 +675,30 @@ intvec* ssiReadIntvec(ssiInfo *d)
   }
   return v;
 }
+intvec* ssiReadIntmat(ssiInfo *d)
+{
+  int r,c;
+  r=s_readint(d->f_read);
+  c=s_readint(d->f_read);
+  intvec *v=new intvec(r,c,0);
+  for(int i=0;i<r*c;i++)
+  {
+    (*v)[i]=s_readint(d->f_read);
+  }
+  return v;
+}
+bigintmat* ssiReadBigintmat(ssiInfo *d)
+{
+  int r,c;
+  r=s_readint(d->f_read);
+  c=s_readint(d->f_read);
+  bigintmat *v=new bigintmat(r,c);
+  for(int i=0;i<r*c;i++)
+  {
+    (*v)[i]=ssiReadBigInt(d);
+  }
+  return v;
+}
 
 void ssiReadBlackbox(leftv res, si_link l)
 {
@@ -702,31 +762,37 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
 
         int pc[2];
         int cp[2];
+        SSI_BLOCK_CHLD;
         pipe(pc);
         pipe(cp);
         pid_t pid=fork();
+        SSI_UNBLOCK_CHLD;
         if (pid==0) /*fork: child*/
         {
           link_list hh=(link_list)ssiToBeClosed->next;
           /* we know: l is the first entry in ssiToBeClosed-list */
           while(hh!=NULL)
           {
+            SI_LINK_SET_CLOSE_P(hh->l);
             ssiInfo *dd=(ssiInfo*)hh->l->data;
+            SSI_BLOCK_CHLD;
             s_close(dd->f_read);
             fclose(dd->f_write);
+            SSI_UNBLOCK_CHLD;
             if (dd->r!=NULL) rKill(dd->r);
             omFreeSize((ADDRESS)dd,(sizeof *dd));
             hh->l->data=NULL;
-            SI_LINK_SET_CLOSE_P(hh->l);
             link_list nn=(link_list)hh->next;
             omFree(hh);
             hh=nn;
           }
           ssiToBeClosed->next=NULL;
+          SSI_BLOCK_CHLD;
           close(pc[1]); close(cp[0]);
+          d->f_write=fdopen(cp[1],"w");
+          SSI_UNBLOCK_CHLD;
           d->f_read=s_open(pc[0]);
           d->fd_read=pc[0];
-          d->f_write=fdopen(cp[1],"w");
           d->fd_write=cp[1];
           l->data=d;
           omFree(l->mode);
@@ -758,10 +824,12 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
         else if (pid>0) /*fork: parent*/
         {
           d->pid=pid;
+          SSI_BLOCK_CHLD;
           close(pc[0]); close(cp[1]);
+          d->f_write=fdopen(pc[1],"w");
+          SSI_UNBLOCK_CHLD;
           d->f_read=s_open(cp[0]);
           d->fd_read=cp[0];
-          d->f_write=fdopen(pc[1],"w");
           d->fd_write=pc[1];
           SI_LINK_SET_RW_OPEN_P(l);
           d->send_quit_at_exit=1;
@@ -908,11 +976,15 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
         d->fd_read = newsockfd;
         d->fd_write = newsockfd;
         d->f_read = s_open(newsockfd);
+        SSI_BLOCK_CHLD;
         d->f_write = fdopen(newsockfd, "w");
+        close(sockfd);
+        SSI_UNBLOCK_CHLD;
         SI_LINK_SET_RW_OPEN_P(l);
         d->send_quit_at_exit=1;
-        close(sockfd);
+        SSI_BLOCK_CHLD;
         fprintf(d->f_write,"98 %d %d %u %u\n",SSI_VERSION,MAX_TOK,test,verbose);
+        SSI_UNBLOCK_CHLD;
       }
       // ----------------------------------------------------------------------
       else if(strcmp(mode,"connect")==0)
@@ -974,18 +1046,24 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
             mode="w";
           }
         }
+        SSI_BLOCK_CHLD;
         outfile=myfopen(filename,mode);
+        SSI_UNBLOCK_CHLD;
         if (outfile!=NULL)
         {
           if (strcmp(l->mode,"r")==0)
           {
+            SSI_BLOCK_CHLD;
             fclose(outfile);
             d->f_read=s_open_by_name(filename);
+            SSI_UNBLOCK_CHLD;
           }
           else
           {
+            SSI_BLOCK_CHLD;
             d->f_write = outfile;
             fprintf(d->f_write,"98 %d %d %u %u\n",SSI_VERSION,MAX_TOK,test,verbose);
+            SSI_UNBLOCK_CHLD;
           }
         }
         else
@@ -1002,6 +1080,26 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
 }
 
 //**************************************************************************/
+BOOLEAN ssiPrepClose(si_link l)
+{
+  if (l!=NULL)
+  {
+    ssiInfo *d = (ssiInfo *)l->data;
+    if (d!=NULL)
+    {
+      if (d->send_quit_at_exit)
+      {
+        SSI_BLOCK_CHLD;
+        fputs("99\n",d->f_write);
+        fflush(d->f_write);
+        SSI_UNBLOCK_CHLD;
+      }
+      d->quit_sent=1;
+    }
+  }
+  return FALSE;
+}
+
 BOOLEAN ssiClose(si_link l)
 {
   if (l!=NULL)
@@ -1010,24 +1108,29 @@ BOOLEAN ssiClose(si_link l)
     ssiInfo *d = (ssiInfo *)l->data;
     if (d!=NULL)
     {
-      if (d->r!=NULL) rKill(d->r);
-      if (d->send_quit_at_exit)
+      if ((d->send_quit_at_exit)
+      && (d->quit_sent==0))
       {
-        fputs("99\n",d->f_write);fflush(d->f_write);
+        SSI_BLOCK_CHLD;
+        fputs("99\n",d->f_write);
+        fflush(d->f_write);
+        SSI_UNBLOCK_CHLD;
       }
-      if (d->pid!=0)
+      if (d->r!=NULL) rKill(d->r);
+      if ((d->pid!=0)
+      && (waitpid(d->pid,NULL,WNOHANG)==0))
       {
         struct timespec t;
         t.tv_sec=0;
-        t.tv_nsec=100000000; // <=100 ms
+        t.tv_nsec=50000000; // <=50 ms
         int r=nanosleep(&t,NULL);
-        if(waitpid(d->pid,NULL,WNOHANG)==0)
+        if((r==0) && (waitpid(d->pid,NULL,WNOHANG)==0))
         {
           kill(d->pid,15);
           t.tv_sec=0;
-          t.tv_nsec=10000000; // <=100 ms
+          t.tv_nsec=10000000; // <=10 ms
           r=nanosleep(&t,NULL);
-          if(waitpid(d->pid,NULL,WNOHANG)==0)
+          if((r==0)&&(waitpid(d->pid,NULL,WNOHANG)==0))
           {
             kill(d->pid,9); // just to be sure
             waitpid(d->pid,NULL,0);
@@ -1148,6 +1251,12 @@ leftv ssiRead1(si_link l)
     case 17: res->rtyp=INTVEC_CMD;
              res->data=ssiReadIntvec(d);
              break;
+    case 18: res->rtyp=INTMAT_CMD;
+             res->data=ssiReadIntmat(d);
+             break;
+    case 19: res->rtyp=BIGINTMAT_CMD;
+             res->data=ssiReadBigintmat(d);
+	     break;
     case 20: ssiReadBlackbox(res,l);
              break;
     // ------------
@@ -1274,6 +1383,14 @@ BOOLEAN ssiWrite(si_link l, leftv data)
                    fputs("17 ",d->f_write);
                    ssiWriteIntvec(d,(intvec *)dd);
                    break;
+          case INTMAT_CMD:
+                   fputs("18 ",d->f_write);
+                   ssiWriteIntmat(d,(intvec *)dd);
+                   break;
+          case BIGINTMAT_CMD:
+                   fputs("19 ",d->f_write);
+                   ssiWriteBigintmat(d,(bigintmat *)dd);
+                   break;
           default:
             if (tt>MAX_TOK)
             {
@@ -1296,6 +1413,9 @@ BOOLEAN ssiWrite(si_link l, leftv data)
   return FALSE;
 }
 
+BOOLEAN ssiGetDump(si_link l);
+BOOLEAN ssiDump(si_link l);
+
 si_link_extension slInitSsiExtension(si_link_extension s)
 {
   s->Open=ssiOpen;
@@ -1304,6 +1424,8 @@ si_link_extension slInitSsiExtension(si_link_extension s)
   s->Read=ssiRead1;
   s->Read2=(slRead2Proc)NULL;
   s->Write=ssiWrite;
+  s->Dump=ssiDump;
+  s->GetDump=ssiGetDump;
 
   s->Status=slStatusSsi;
   s->type="ssi";
@@ -1751,33 +1873,153 @@ void sig_chld_hdl(int sig)
   pid_t kidpid;
   int status;
 
-  while ((kidpid = waitpid(-1, &status, WNOHANG)) > 0)
+  loop
   {
-     //printf("Child %ld terminated\n", kidpid);
-     link_list hh=ssiToBeClosed;
-     while(hh!=NULL)
-     {
-       if((hh->l!=NULL) && (hh->l->m->Open==ssiOpen))
-       {
-          ssiInfo *d = (ssiInfo *)hh->l->data;
-          if(d->pid==kidpid)
+    kidpid = waitpid(-1, &status, WNOHANG);
+    if (kidpid==-1)
+    {
+      /* continue on interruption (EINTR): */
+      if (errno == EINTR) continue;
+      /* break on anything else (EINVAL or ECHILD according to manpage): */
+      break;
+    }
+    else if (kidpid==0) break; /* no more children to process, so break */
+
+    //printf("Child %ld terminated\n", kidpid);
+    link_list hh=ssiToBeClosed;
+    while(hh!=NULL)
+    {
+      if((hh->l!=NULL) && (hh->l->m->Open==ssiOpen))
+      {
+        ssiInfo *d = (ssiInfo *)hh->l->data;
+        if(d->pid==kidpid)
+        {
+          if(ssiToBeClosed_inactive)
           {
-            if(ssiToBeClosed_inactive)
-            {
-              ssiToBeClosed_inactive=FALSE;
-              slClose(hh->l);
-              ssiToBeClosed_inactive=TRUE;
-              break;
-            }
-            else break;
+            ssiToBeClosed_inactive=FALSE;
+            slClose(hh->l);
+            ssiToBeClosed_inactive=TRUE;
+            break;
           }
-          else hh=(link_list)hh->next;
-       }
-       else hh=(link_list)hh->next;
-     }
+          else break;
+        }
+        else hh=(link_list)hh->next;
+      }
+      else hh=(link_list)hh->next;
+    }
   }
 }
 
+static BOOLEAN DumpSsiIdhdl(si_link l, idhdl h)
+{
+  int type_id = IDTYP(h);
+
+  // C-proc not to be dumped, also LIB-proc not
+  if (type_id == PROC_CMD)
+  {
+    if (IDPROC(h)->language == LANG_C) return FALSE;
+    if (IDPROC(h)->libname != NULL) return FALSE;
+  }
+  // do not dump links
+  if (type_id == LINK_CMD) return FALSE;
+
+  // do not dump ssi internal rings: ssiRing*
+  if ((type_id == RING_CMD) && (strncmp(IDID(h),"ssiRing",7)==0))
+    return FALSE;
+
+  command D=(command)omAlloc0(sizeof(*D));
+  sleftv tmp;
+  memset(&tmp,0,sizeof(tmp));
+  tmp.rtyp=COMMAND;
+  tmp.data=D;
+
+  if (type_id == PACKAGE_CMD)
+  {
+    // do not dump Top
+    if (strcmp(IDID(h), "Top") == 0) return FALSE;
+    package p=(package)IDDATA(h);
+    // dump Singular-packages as load("...");
+    if (p->language==LANG_SINGULAR)
+    {
+      D->op=LOAD_CMD;
+      D->argc=1;
+      D->arg1.rtyp=STRING_CMD;
+      D->arg1.data=p->libname;
+      ssiWrite(l,&tmp);
+      omFree(D);
+      return FALSE;
+    }
+  }
+
+  // handle qrings separately
+  //if (type_id == QRING_CMD)
+  //  return DumpSsiQringQring(l, h);
+
+  // put type and name
+  //Print("generic dump:%s,%s\n",IDID(h),Tok2Cmdname(IDTYP(h)));
+  D->op='=';
+  D->argc=2;
+  D->arg1.rtyp=DEF_CMD;
+  D->arg1.name=IDID(h);
+  D->arg2.rtyp=IDTYP(h);
+  D->arg2.data=IDDATA(h);
+  ssiWrite(l,&tmp);
+  omFree(D);
+  return FALSE;
+}
+static BOOLEAN ssiDumpIter(si_link l, idhdl h)
+{
+  if (h == NULL) return FALSE;
+
+  if (ssiDumpIter(l, IDNEXT(h))) return TRUE;
+
+  // need to set the ring before writing it, otherwise we get in
+  // trouble with minpoly
+  if (IDTYP(h) == RING_CMD || IDTYP(h) == QRING_CMD)
+    rSetHdl(h);
+
+  if (DumpSsiIdhdl(l, h)) return TRUE;
+
+  // do not dump ssi internal rings: ssiRing*
+  // but dump objects of all other rings
+  if ((IDTYP(h) == RING_CMD || IDTYP(h) == QRING_CMD)
+  && (strncmp(IDID(h),"ssiRing",7)!=0))
+    return ssiDumpIter(l, IDRING(h)->idroot);
+  else
+    return FALSE;
+}
+BOOLEAN ssiDump(si_link l)
+{
+  idhdl h = IDROOT, rh = currRingHdl;
+  BOOLEAN status = ssiDumpIter(l, h);
+
+  //if (! status ) status = DumpAsciiMaps(fd, h, NULL);
+
+  if (currRingHdl != rh) rSetHdl(rh);
+  //fprintf(fd, "option(set, intvec(%d, %d));\n", test, verbose);
+
+  return status;
+}
+BOOLEAN ssiGetDump(si_link l)
+{
+  ssiInfo *d=(ssiInfo*)l->data;
+  loop
+  {
+    if (!SI_LINK_OPEN_P(l)) break;
+    if (s_iseof(d->f_read)) break;
+    leftv h=ssiRead1(l); /*contains an exit.... */
+    if (feErrors != NULL && *feErrors != '\0')
+    {
+      // handle errors:
+      PrintS(feErrors); /* currently quite simple */
+      return TRUE;
+      *feErrors = '\0';
+    }
+    h->CleanUp();
+    omFreeBin(h, sleftv_bin);
+  }
+  return FALSE;
+}
 // ----------------------------------------------------------------
 // format
 // 1 int %d
@@ -1797,6 +2039,8 @@ void sig_chld_hdl(int sig)
 // 15 setring .......
 // 16 nothing
 // 17 intvec <len> ...
+// 18 intmat
+// 19 bigintmat <r> <c> ...
 //
 // 20 blackbox <name> ...
 //
