@@ -93,8 +93,7 @@ void sleftv::Print(leftv store, int spaces)
     const char *n=Name();
     char *s;
     void *d=Data();
-    if (errorreported)
-      return;
+    if (errorreported) return;
     if ((store!=NULL)&&(store!=this))
       store->CleanUp();
 
@@ -294,7 +293,9 @@ void sleftv::CleanUp(ring r)
   if ((name!=NULL) && (name!=sNoName) && (rtyp!=IDHDL) && (rtyp!=ALIAS_CMD))
   {
     //::Print("free %x (%s)\n",name,name);
-    omFreeBinAddr((ADDRESS)name);
+    //omFreeBinAddr((ADDRESS)name);
+    omFree((ADDRESS)name); /* here, a name (monomial) may really be larger
+                              then 1000 chars => not a BinAddr */
   }
   //name=NULL;
   //flag=0;
@@ -678,9 +679,6 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
   if (!errorreported)
   {
     char *s;
-    const char *n;
-    if (name!=NULL) n=name;
-    else n=sNoName;
     int t=Typ();
     switch (t /*Typ()*/)
     {
@@ -1427,7 +1425,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
       /* 3) existing identifier, local */
       if ((h!=NULL) && (IDLEV(h)==myynest))
       {
-        if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id);
+        if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
         goto id_found;
       }
     }
@@ -1454,7 +1452,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
     /* 5. existing identifier, global */
     if (h!=NULL)
     {
-      if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id);
+      if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
       goto id_found;
     }
     /* 6. local ring: number/poly */
@@ -1473,7 +1471,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
           // in this case we may have monomials equal to 0 in p_Read
           v->name = id;
           #else
-          omFreeBinAddr((ADDRESS)id);
+          omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
           #endif
         }
         else if (pIsConstant(p))
@@ -1511,7 +1509,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
           // in this case we may have monomials equal to 0 in p_Read
           v->name = id;
           #else
-          omFreeBinAddr((ADDRESS)id);
+          omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
           #endif
         }
         else
@@ -1537,7 +1535,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
     {
       if (strcmp(id,IDID(currRingHdl))==0)
       {
-        if (IDID(currRingHdl)!=id) omFreeBinAddr((ADDRESS)id);
+        if (IDID(currRingHdl)!=id) omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
         h=currRingHdl;
         goto id_found;
       }
@@ -1547,7 +1545,7 @@ void syMake(leftv v,const char * id, idhdl packhdl)
       h=basePack->idroot->get(id,myynest);
       if (h!=NULL)
       {
-        if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id);
+        if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
         v->req_packhdl=basePack;
         goto id_found;
       }
@@ -1616,11 +1614,9 @@ int sleftv::Eval()
         nok=d->arg2.Eval();
         if(!nok)
         {
-          leftv r=iiMake_proc(h,req_packhdl,&d->arg2);
-          if (r!=NULL)
-            memcpy(this,r,sizeof(sleftv));
-          else
-            nok=TRUE;
+          nok=iiMake_proc(h,req_packhdl,&d->arg2);
+          if (!nok)
+            memcpy(this,&iiRETURNEXPR,sizeof(sleftv));
         }
       }
       else nok=TRUE;
