@@ -9,8 +9,10 @@
 #include <kernel/longrat.h>
 #include <Singular/subexpr.h>
 #include <Singular/ipshell.h>
+#include <Singular/lists.h>
 #include <sstream>
 #include <gfanlib/gfanlib.h>
+#include <gfanlib/gfanlib_zfan.h>
 
 #include <callgfanlib/bbfan.h>
 #include <callgfanlib/bbcone.h>
@@ -18,9 +20,9 @@
 
 int fanID;
 
-void *bbfan_Init(blackbox *b)
+void* bbfan_Init(blackbox *b)
 {
-  return (void*)(new gfan::ZFan(0));
+  return (void*) new gfan::ZFan(0);
 }
 
 void bbfan_destroy(blackbox *b, void *d)
@@ -32,7 +34,7 @@ void bbfan_destroy(blackbox *b, void *d)
   }
 }
 
-char * bbfan_String(blackbox *b, void *d)
+char* bbfan_String(blackbox *b, void *d)
 {
   if (d==NULL) return omStrDup("invalid object");
   else
@@ -43,7 +45,7 @@ char * bbfan_String(blackbox *b, void *d)
   }
 }
 
-void * bbfan_Copy(blackbox*b, void *d)
+void* bbfan_Copy(blackbox*b, void *d)
 {
   gfan::ZFan* zf = (gfan::ZFan*)d;
   gfan::ZFan* newZf = new gfan::ZFan(*zf);
@@ -57,7 +59,7 @@ BOOLEAN bbfan_Assign(leftv l, leftv r)
   {
     if (l->Data()!=NULL)
     {
-      gfan::ZFan* zd = (gfan::ZFan*)l->Data();
+      gfan::ZFan* zd = (gfan::ZFan*) l->Data();
       delete zd;
     }
     newZf = new gfan::ZFan(0);
@@ -66,15 +68,15 @@ BOOLEAN bbfan_Assign(leftv l, leftv r)
   {
     if (l->Data()!=NULL)
     {
-      gfan::ZFan* zd = (gfan::ZFan*)l->Data();
+      gfan::ZFan* zd = (gfan::ZFan*) l->Data();
       delete zd;
     }
-    gfan::ZFan* zf = (gfan::ZFan*)r->Data();
+    gfan::ZFan* zf = (gfan::ZFan*) r->Data();
     newZf = new gfan::ZFan(*zf);
   }
   else if (r->Typ()==INT_CMD)
   {
-    int ambientDim = (int)(long)r->Data();
+    int ambientDim = (int) (long) r->Data();
     if (ambientDim < 0)
     {
       Werror("expected an int >= 0, but got %d", ambientDim);
@@ -82,7 +84,7 @@ BOOLEAN bbfan_Assign(leftv l, leftv r)
     }
     if (l->Data()!=NULL)
     {
-      gfan::ZFan* zd = (gfan::ZFan*)l->Data();
+      gfan::ZFan* zd = (gfan::ZFan*) l->Data();
       delete zd;
     }
     newZf = new gfan::ZFan(ambientDim);
@@ -95,11 +97,11 @@ BOOLEAN bbfan_Assign(leftv l, leftv r)
 
   if (l->rtyp==IDHDL)
   {
-    IDDATA((idhdl)l->data)=(char *)newZf;
+    IDDATA((idhdl)l->data) = (char*) newZf;
   }
   else
   {
-    l->data=(void *)newZf;
+    l->data = (void*) newZf;
   }
   return FALSE;
 }
@@ -109,22 +111,22 @@ BOOLEAN bbfan_Assign(leftv l, leftv r)
    bigintmat; 0 otherwise */
 static gfan::IntMatrix permutationIntMatrix(const bigintmat* iv)
 {
-        int cc = iv->cols();
-        int rr = iv->rows();
-        bigintmat* ivCopy = new bigintmat(rr, cc);
-        for (int r = 1; r <= rr; r++)
-          for (int c = 1; c <= cc; c++)
-	  {
-	    number temp1 = nlInit(1,NULL);
-	    number temp2 = nlSub(IMATELEM(*iv, r, c),temp1);
-	    ivCopy->set(r,c,temp2);
-	    nlDelete(&temp1,NULL); 
-	    nlDelete(&temp2,NULL);
-	  }
-        gfan::ZMatrix* zm = bigintmatToZMatrix(ivCopy);
-        gfan::IntMatrix* im = new gfan::IntMatrix(gfan::ZToIntMatrix(*zm));
-        delete zm;
-        return *im;
+  int cc = iv->cols();
+  int rr = iv->rows();
+  bigintmat* ivCopy = new bigintmat(rr, cc);
+  for (int r = 1; r <= rr; r++)
+    for (int c = 1; c <= cc; c++)
+    {
+      number temp1 = nlInit(1,NULL);
+      number temp2 = nlSub(IMATELEM(*iv, r, c),temp1);
+      ivCopy->set(r,c,temp2);
+      nlDelete(&temp1,NULL); 
+      nlDelete(&temp2,NULL);
+    }
+  gfan::ZMatrix* zm = bigintmatToZMatrix(ivCopy);
+  gfan::IntMatrix im = gfan::IntMatrix(gfan::ZToIntMatrix(*zm));
+  delete zm;
+  return im;
 }
 static BOOLEAN jjFANEMPTY_I(leftv res, leftv v)
 {
@@ -135,7 +137,7 @@ static BOOLEAN jjFANEMPTY_I(leftv res, leftv v)
     return TRUE;
   }
   res->rtyp = fanID;
-  res->data = (char*)(new gfan::ZFan(ambientDim));
+  res->data = (void*)(new gfan::ZFan(ambientDim));
   return FALSE;
 }
 
@@ -152,7 +154,7 @@ static BOOLEAN jjFANEMPTY_IM(leftv res, leftv v)
   gfan::SymmetryGroup sg = gfan::SymmetryGroup(ambientDim);
   sg.computeClosure(im);
   res->rtyp = fanID;
-  res->data = (char*)(new gfan::ZFan(sg));
+  res->data = (void*)(new gfan::ZFan(sg));
   return FALSE;
 }
 
@@ -162,7 +164,7 @@ BOOLEAN emptyFan(leftv res, leftv args)
   if (u == NULL)
   {
     res->rtyp = fanID;
-    res->data = (void*)(new gfan::ZFan(0));
+    res->data = (void*) new gfan::ZFan(0);
     return FALSE;
   }
   if ((u != NULL) && (u->Typ() == INT_CMD))
@@ -187,7 +189,7 @@ static BOOLEAN jjFANFULL_I(leftv res, leftv v)
   }
   gfan::ZFan* zf = new gfan::ZFan(gfan::ZFan::fullFan(ambientDim));
   res->rtyp = fanID;
-  res->data = (char*)zf;
+  res->data = (void*) zf;
   return FALSE;
 }
 static BOOLEAN jjFANFULL_IM(leftv res, leftv v)
@@ -204,35 +206,23 @@ static BOOLEAN jjFANFULL_IM(leftv res, leftv v)
   sg.computeClosure(im);
   gfan::ZFan* zf = new gfan::ZFan(gfan::ZFan::fullFan(sg));
   res->rtyp = fanID;
-  res->data = (char*)zf;
+  res->data = (void*) zf;
   return FALSE;
 }
 
 BOOLEAN fullFan(leftv res, leftv args)
 {
-  /*  {
-    gfan::ZFan f(2);
-    std::cout<<f.toString();
-    f.insert(gfan::ZCone::positiveOrthant(2));
-    std::cout<<f.toString();
-    }*/
-
-
   leftv u = args;
   if (u == NULL)
   {
     res->rtyp = fanID;
-    res->data = (void*)(new gfan::ZFan(0));
+    res->data = (void*) new gfan::ZFan(0);
     return FALSE;
   }
   if ((u != NULL) && (u->Typ() == INT_CMD))
-  {
     if (u->next == NULL) return jjFANFULL_I(res, u);
-  }
   if ((u != NULL) && (u->Typ() == BIGINTMAT_CMD))
-  {
     if (u->next == NULL) return jjFANFULL_IM(res, u);
-  }
   WerrorS("fullFan: unexpected parameters");
   return TRUE;
 }
@@ -261,33 +251,45 @@ BOOLEAN numberOfConesOfDimension(leftv res, leftv args)
 {
   leftv u=args;
   if ((u != NULL) && (u->Typ() == fanID))
+  {
+    leftv v=u->next;
+    if ((v != NULL) && (v->Typ() == INT_CMD))
     {
-      leftv v=u->next;
-      if ((v != NULL) && (v->Typ() == INT_CMD))
+      leftv w=v->next;
+      if ((w != NULL) && (w->Typ() == INT_CMD))
+      {
+        leftv x=w->next;
+        if ((x != NULL) && (x->Typ() == INT_CMD))
         {
-          leftv w=v->next;
-          if ((w != NULL) && (w->Typ() == INT_CMD))
+          gfan::ZFan* zf = (gfan::ZFan*) u->Data();
+          int d = (int)(long)v->Data();
+          int o = (int)(long)w->Data();
+          int m = (int)(long)x->Data();
+          if ( (0<=d) && (d <= zf->getAmbientDimension()) 
+                      && ((o == 0) || (o == 1)) 
+                      && ((m == 0) || (m == 1)))
+          {
+            bool oo = (bool) o;
+            bool mm = (bool) m;
+            int ld = zf->getLinealityDimension();
+            if (d-ld>=0)
             {
-              leftv x=w->next;
-              if ((x != NULL) && (x->Typ() == INT_CMD))
-                {
-                  gfan::ZFan* zf = (gfan::ZFan*) u->Data();
-                  int d = (int)(long)v->Data();
-                  int o = (int)(long)w->Data();
-                  int m = (int)(long)x->Data();
-                  if ((d <= zf->getAmbientDimension()) && ((o == 0) || (o == 1)) && ((m == 0) || (m == 1)))
-                    {
-                      bool oo = (bool) o;
-                      bool mm = (bool) m;
-                      int n = zf->numberOfConesOfDimension(d,oo,mm);
-                      res->rtyp = INT_CMD;
-                      res->data = (char*) n;
-                      return FALSE;
-                    }
-                }
+              int n = zf->numberOfConesOfDimension(d,oo,mm);
+              res->rtyp = INT_CMD;
+              res->data = (void*) (long) n;
+              return FALSE;
             }
+            else
+            {
+              res->rtyp = INT_CMD;
+              res->data = (void*) (long) 0;
+              return FALSE;
+            }
+          }
         }
+      }
     }
+  }
   WerrorS("numberOfConesOfDimension: unexpected parameters");
   return TRUE;
 }
@@ -305,7 +307,7 @@ BOOLEAN ncones(leftv res, leftv args)
         { n = n + zf->numberOfConesOfDimension(i,0,0); }
 
       res->rtyp = INT_CMD;
-      res->data = (char*) n;
+      res->data = (void*) (long) n;
       return FALSE;
     }
   else
@@ -327,7 +329,7 @@ BOOLEAN nmaxcones(leftv res, leftv args)
         { n = n + zf->numberOfConesOfDimension(d,0,1); }
 
       res->rtyp = INT_CMD;
-      res->data = (char*) n;
+      res->data = (void*) (long) n;
       return FALSE;
     }
   else
@@ -337,7 +339,7 @@ BOOLEAN nmaxcones(leftv res, leftv args)
     }
 }
 
-bool iscompatible(gfan::ZFan* zf, gfan::ZCone* zc)
+bool isCompatible(gfan::ZFan* zf, gfan::ZCone* zc)
 {
   bool b = (zf->getAmbientDimension() == zc->ambientDimension());
   if(b)
@@ -366,13 +368,12 @@ BOOLEAN isCompatible(leftv res, leftv args)
     {
       gfan::ZFan* zf = (gfan::ZFan*)u->Data();
       gfan::ZCone* zc = (gfan::ZCone*)v->Data();
-      bool b = iscompatible(zf,zc);
-      int bb = (int) b;
+      int b = isCompatible(zf,zc);
       res->rtyp = INT_CMD;
-      res->data = (char*) bb;
+      res->data = (void*) (long) b;
       return FALSE;
-      }
     }
+  }
   WerrorS("isCompatible: unexpected parameters");
   return TRUE;
 }
@@ -396,7 +397,7 @@ BOOLEAN insertCone(leftv res, leftv args)
 
       if (n != 0)
       {
-        if (!iscompatible(zf,zc))
+        if (!isCompatible(zf,zc))
         {          
           WerrorS("insertCone: cone and fan not compatible");
           return TRUE;
@@ -406,7 +407,7 @@ BOOLEAN insertCone(leftv res, leftv args)
       zf->insert(*zc);
       res->rtyp = NONE;
       res->data = NULL;
-      IDDATA((idhdl)u->data)=(char *)zf;
+      IDDATA((idhdl)u->data) = (char*) zf;
       return FALSE;
     }
   }
@@ -419,24 +420,22 @@ BOOLEAN insertCone(leftv res, leftv args)
 
 bool containsInCollection(gfan::ZFan* zf, gfan::ZCone* zc)
 {
-  if((zf->getAmbientDimension() == zc->ambientDimension()))
+  gfan::ZVector zv=zc->getRelativeInteriorPoint();
+  for (int d=0; d<=zf->getAmbientDimension(); d++)
   {
-    gfan::ZVector zv=zc->getRelativeInteriorPoint();
-    for (int d=0; d<=zf->getAmbientDimension(); d++)
+    for (int i=0; i<zf->numberOfConesOfDimension(d,0,1); i++)
     {
-      for (int i=0; i<zf->numberOfConesOfDimension(d,0,1); i++)
+      gfan::ZCone zd = zf->getCone(d,i,0,1);
+      zd.canonicalize();
+      if (zd.containsRelatively(zv))
       {
-        gfan::ZCone zd = zf->getCone(d,i,0,1);
-        zd.canonicalize();
-        if (zd.containsRelatively(zv))
-        {
-          gfan::ZCone temp = *zc;
-          temp.canonicalize();
-          return (!(zd != temp));
-        }
+        gfan::ZCone temp = *zc;
+        temp.canonicalize();
+        return (!(zd != temp));
       }
     }
   }
+  return 0;
 }
 
 BOOLEAN containsInCollection(leftv res, leftv args)
@@ -449,9 +448,14 @@ BOOLEAN containsInCollection(leftv res, leftv args)
     {
       gfan::ZFan* zf = (gfan::ZFan*)u->Data();
       gfan::ZCone* zc = (gfan::ZCone*)v->Data();
-      res->rtyp = INT_CMD;
-      res->data = (char*) (int) containsInCollection(zf,zc);
-      return FALSE;
+      if((zf->getAmbientDimension() == zc->ambientDimension()))
+      {
+        res->rtyp = INT_CMD;
+        res->data = (void*) (long) (int) containsInCollection(zf,zc);
+        return FALSE;
+      }
+      WerrorS("containsInCollection: mismatching ambient dimensions");
+      return TRUE;
     }
   }
   // if ((u != NULL) && (u->Typ() == coneID))
@@ -462,7 +466,7 @@ BOOLEAN containsInCollection(leftv res, leftv args)
   //     gfan::ZCone* zc = (gfan::ZCone*)u->Data();
   //     gfan::ZCone* zd = (gfan::ZCone*)v->Data();
   //     res->rtyp = INT_CMD;
-  //     res->data = (char*) (int) hasFace(zc,zd);
+  //     res->data = (void*) (int) hasFace(zc,zd);
   //     return FALSE;
   //   }
   // }
@@ -513,7 +517,7 @@ BOOLEAN removeCone(leftv res, leftv args)
       zf->remove(*zc);
       res->rtyp = NONE;
       res->data = NULL;
-      IDDATA((idhdl)u->data)=(char *)zf;
+      IDDATA((idhdl)u->data) = (char*) zf;
       return FALSE;
     }
   }
@@ -532,49 +536,62 @@ BOOLEAN getCone(leftv res, leftv args)
       leftv w=v->next;
       if ((w != NULL) && (w->Typ() == INT_CMD))
       {
+        gfan::ZFan* zf = (gfan::ZFan*) u->Data();
+        int d = (int)(long)v->Data();
+        int i = (int)(long)w->Data();
+        int o = -1;
+        int m = -1;
         leftv x=w->next;
         if ((x != NULL) && (x->Typ() == INT_CMD))
         {
-          leftv y=w->next;
+          o = (int)(long)x->Data();
+          leftv y=x->next;
           if ((y != NULL) && (y->Typ() == INT_CMD))
           {
-            gfan::ZFan* zf = (gfan::ZFan*) u->Data();
-            int d = (int)(long)v->Data();
-            int i = (int)(long)w->Data();
-            int o = (int)(long)x->Data();
-            int m = (int)(long)y->Data();
-            if (((o == 0) || (o == 1)) && ((m == 0) || (m == 1)))
+            m = (int)(long)y->Data();
+          }
+        }
+        if (o == -1) o = 0;
+        if (m == -1) m = 0;
+        if (((o == 0) || (o == 1)) && ((m == 0) || (m == 1)))
+        {
+          bool oo = (bool) o;
+          bool mm = (bool) m;
+          if (0<=d && d<=zf->getAmbientDimension())
+          {
+            if (0<i && i<=zf->numberOfConesOfDimension(d,oo,mm))
             {
-              bool oo = (bool) o;
-              bool mm = (bool) m;
-              if (d<=zf->getAmbientDimension())
+              i=i-1;
+              int ld = zf->getLinealityDimension();
+              if (d-ld>=0)
               {
-                if (i<=zf->numberOfConesOfDimension(d,oo,mm))
-                {
-                  i=i-1;
-                  gfan::ZCone zc = zf->getCone(d,i,oo,mm);
-                  res->rtyp = coneID;
-                  res->data = (char*)new gfan::ZCone(zc);
-                  return FALSE;
-                }
-                else
-                {
-                  WerrorS("getCone: invalid index");
-                  return TRUE;
-               }
-             }
-             else
-             {
-               WerrorS("getCone: invalid dimension");
-               return TRUE;
+                gfan::ZCone zc = zf->getCone(d-ld,i,oo,mm);
+                res->rtyp = coneID;
+                res->data = (void*)new gfan::ZCone(zc);
+                return FALSE;
+              }
+              else
+              {
+                WerrorS("getCone: invalid dimension; no cones in this dimension");
+                return TRUE;
               }
             }
             else
             {
-              WerrorS("getCone: invalid specifier for orbit or maximal");
+              WerrorS("getCone: invalid index");
               return TRUE;
             }
           }
+          else
+          {
+            WerrorS("getCone: invalid dimension");
+            return TRUE;
+          }
+        }
+        else
+        {
+          WerrorS("getCone: invalid specifier for orbit or maximal");
+          return TRUE;
         }
       }
     }
@@ -586,27 +603,81 @@ BOOLEAN getCone(leftv res, leftv args)
   }
 }
 
-BOOLEAN isSimplicial(leftv res, leftv args)
+BOOLEAN getCones(leftv res, leftv args)
 {
   leftv u=args;
   if ((u != NULL) && (u->Typ() == fanID))
   {
-    gfan::ZFan* zf = (gfan::ZFan*) u->Data();
-    bool b=zf->isSimplicial();
-    res->rtyp = INT_CMD;
-    res->data = (char*) (int) b;
-    return FALSE;
-  }
-  if ((u != NULL) && (u->Typ() == coneID))
+    leftv v=u->next;
+    if ((v != NULL) && (v->Typ() == INT_CMD))
     {
-      gfan::ZCone* zc = (gfan::ZCone*) u->Data();
-      int b = isSimplicial(zc);
-      res->rtyp = INT_CMD;
-      res->data = (char*) b;
-      return FALSE;
+      gfan::ZFan* zf = (gfan::ZFan*) u->Data();
+      int d = (int)(long)v->Data();
+      int o = -1;
+      int m = -1;
+      leftv w=v->next;
+      if ((w != NULL) && (w->Typ() == INT_CMD))
+      {
+        o = (int)(long)w->Data();
+        leftv x=w->next;
+        if ((x != NULL) && (x->Typ() == INT_CMD))
+        {
+          m = (int)(long)x->Data();
+        }
+      }
+      if (o == -1) o = 0;
+      if (m == -1) m = 0;
+      if (((o == 0) || (o == 1)) && ((m == 0) || (m == 1)))
+      {
+        bool oo = (bool) o;
+        bool mm = (bool) m;
+        if (0<=d && d<=zf->getAmbientDimension())
+        {
+          int ld = zf->getLinealityDimension();
+          if (d-ld>=0)
+          {
+            lists L = (lists)omAllocBin(slists_bin);
+            int n = zf->numberOfConesOfDimension(d,oo,mm);
+            L->Init(n);
+            for (int i=0; i<n; i++)
+            {
+              gfan::ZCone zc = zf->getCone(d-ld,i,oo,mm);
+              L->m[i].rtyp = coneID; L->m[i].data=(void*) new gfan::ZCone(zc);
+            }
+            res->rtyp = LIST_CMD;
+            res->data = (void*) L;
+            return FALSE;
+          }
+          else
+          {
+            WerrorS("getCones: invalid dimension; no cones in this dimension");
+            return TRUE;
+          }
+        }
+        else
+        {
+          WerrorS("getCones: invalid dimension");
+          return TRUE;
+        }
+      }
+      else
+      {
+        WerrorS("getCones: invalid specifier for orbit or maximal");
+        return TRUE;
+      }
     }
-  WerrorS("isSimplicial: unexpected parameters");
-  return TRUE;
+  }
+  else
+  {
+    WerrorS("getCones: unexpected parameters");
+    return TRUE;
+  }
+}
+
+int isSimplicial(gfan::ZFan* zf)
+{
+  int i = zf->isSimplicial() ? 1 : 0;
+  return i;
 }
 
 BOOLEAN isPure(leftv res, leftv args)
@@ -615,9 +686,9 @@ BOOLEAN isPure(leftv res, leftv args)
   if ((u != NULL) && (u->Typ() == fanID))
   {
     gfan::ZFan* zf = (gfan::ZFan*) u->Data();
-    bool b=zf->isPure();
+    int b = zf->isPure();
     res->rtyp = INT_CMD;
-    res->data = (char*) (int) b;
+    res->data = (void*) (long) b;
     return FALSE;
   }
   WerrorS("isPure: unexpected parameters");
@@ -628,13 +699,13 @@ BOOLEAN isComplete(leftv res, leftv args)
 {
   leftv u=args;
   if ((u != NULL) && (u->Typ() == fanID))
-    {
-      gfan::ZFan* zf = (gfan::ZFan*) u->Data();
-      bool b=zf->isComplete();
-      res->rtyp = INT_CMD;
-      res->data = (char*) (int) b;
-      return FALSE;
-    }
+  {
+    gfan::ZFan* zf = (gfan::ZFan*) u->Data();
+    int b = zf->isComplete();
+    res->rtyp = INT_CMD;
+    res->data = (void*) (long) b;
+    return FALSE;
+  }
   WerrorS("isComplete: unexpected parameters");
   return TRUE;
 }
@@ -643,13 +714,13 @@ BOOLEAN fVector(leftv res, leftv args)
 {
   leftv u=args;
   if ((u != NULL) && (u->Typ() == fanID))
-    {
-      gfan::ZFan* zf = (gfan::ZFan*) u->Data();
-      gfan::ZVector zv=zf->getFVector();
-      res->rtyp = BIGINTMAT_CMD;
-      res->data = (void*) zVectorToBigintmat(zv);
-      return FALSE;
-    }
+  {
+    gfan::ZFan* zf = (gfan::ZFan*) u->Data();
+    gfan::ZVector zv=zf->getFVector();
+    res->rtyp = BIGINTMAT_CMD;
+    res->data = (void*) zVectorToBigintmat(zv);
+    return FALSE;
+  }
   WerrorS("fVector: unexpected parameters");
   return TRUE;
 }
@@ -666,6 +737,190 @@ gfan::ZMatrix rays(gfan::ZFan* zf)
   return rays;
 }
 
+int numberOfConesWithVector(gfan::ZFan* zf, gfan::ZVector* v)
+{
+  int count = 0;
+  int ambientDim = zf->getAmbientDimension();
+  for (int i=0; i<zf->numberOfConesOfDimension(ambientDim, 0, 0); i++)
+  {
+    gfan::ZCone zc = zf->getCone(ambientDim, i, 0, 0);
+    if (zc.contains(*v))
+    {
+      count = count +1;
+      if (count > 1) 
+        return count;
+    }
+  }
+  return count;
+}
+
+BOOLEAN numberOfConesWithVector(leftv res, leftv args)
+{
+  leftv u=args;
+  if ((u != NULL) && (u->Typ() == fanID))
+  {
+    leftv v=u->next;
+    if ((v != NULL) && (v->Typ() == BIGINTMAT_CMD))
+    {
+      gfan::ZFan* zf = (gfan::ZFan*) u->Data();
+      bigintmat* v0 = (bigintmat*) v->Data();
+      int ambientDim = zf->getAmbientDimension();
+      if (ambientDim != v0->cols())
+      {
+        WerrorS("numberOfConesWithVector: mismatching dimensions");
+        return TRUE;
+      }
+      gfan::ZVector* v1 = bigintmatToZVector(*v0); 
+      int count = numberOfConesWithVector(zf, v1);
+      delete v1;
+      res->rtyp = INT_CMD;
+      res->data = (void*) (long) count;
+      return FALSE;
+    }
+  }
+  WerrorS("numberOfConesWithVector: unexpected parameters");
+  return TRUE;
+}
+
+BOOLEAN fanFromString(leftv res, leftv args)
+{
+  leftv u=args;
+  if ((u != NULL) && (u->Typ() == STRING_CMD))
+    {
+      std::string fanInString = (char*) u->Data();
+      std::istringstream s(fanInString);
+      gfan::ZFan* zf = new gfan::ZFan(s);
+      res->rtyp = fanID;
+      res->data = (void*) zf;
+      return FALSE;
+    }
+  WerrorS("fanFromString: unexpected parameters");
+  return TRUE;
+}
+
+BOOLEAN fanViaCones(leftv res, leftv args)
+{
+  leftv u=args;
+  if ((u != NULL) && (u->Typ() == LIST_CMD))
+  {
+    lists L = (lists) u->Data();
+    if (lSize(L)>-1)
+    {
+      if (L->m[0].Typ() != coneID)
+      {
+        WerrorS("fanViaCones: list contains entries of wrong type");
+        return TRUE;
+      }
+      gfan::ZCone* zc = (gfan::ZCone*) L->m[0].Data();
+      gfan::ZFan* zf = new gfan::ZFan(zc->ambientDimension());
+      zf->insert(*zc);
+      for (int i=1; i<=lSize(L); i++)
+      {
+        if (L->m[i].Typ() != coneID)
+        {
+          WerrorS("fanViaCones: entries of wrong type in list");
+          return TRUE;
+        }
+        gfan::ZCone* zc = (gfan::ZCone*) L->m[i].Data();
+        if (zc->ambientDimension() != zf->getAmbientDimension())
+        {
+          WerrorS("fanViaCones: inconsistent ambient dimensions amongst cones in list");
+          return TRUE;
+        }
+        zf->insert(*zc);
+      }
+      res->rtyp = fanID;
+      res->data = (void*) zf;
+      return FALSE;
+    }
+    res->rtyp = fanID;
+    res->data = (void*) new gfan::ZFan(0);
+    return FALSE;
+  }
+  if ((u != NULL) && (u->Typ() == coneID))
+  {
+    gfan::ZCone* zc = (gfan::ZCone*) u->Data();
+    gfan::ZFan* zf = new gfan::ZFan(zc->ambientDimension());
+    zf->insert(*zc);
+    while (u->next != NULL)
+    {
+      u = u->next;
+      if (u->Typ() != coneID)
+      {
+        WerrorS("fanViaCones: arguments of wrong type");
+        return TRUE;
+      }
+      gfan::ZCone* zc = (gfan::ZCone*) u->Data();
+      if (zc->ambientDimension() != zf->getAmbientDimension())
+      {
+        WerrorS("fanViaCones: inconsistent ambient dimensions amongst input cones");
+        return TRUE;
+      }
+      zf->insert(*zc);
+    }
+    res->rtyp = fanID;
+    res->data = (void*) zf;
+    return FALSE;
+  }
+  if (u == NULL)
+  {
+    res->rtyp = fanID;
+    res->data = (void*) new gfan::ZFan(0);
+    return FALSE;
+  }
+  WerrorS("fanViaCones: unexpected parameters");
+  return TRUE;
+}
+
+lists listOfFacets(const gfan::ZCone &zc)
+{
+  gfan::ZMatrix inequalities = zc.getFacets();
+  gfan::ZMatrix equations = zc.getImpliedEquations();
+  lists L = (lists)omAllocBin(slists_bin);
+  int r = inequalities.getHeight();
+  int c = inequalities.getWidth();
+  L->Init(r);
+  
+  /* next we iterate over each of the r facets, build the respective cone and add it to the list */
+  /* this is the i=0 case */
+  gfan::ZMatrix newInequalities = inequalities.submatrix(1,0,r,c);
+  gfan::ZMatrix newEquations = equations; 
+  newEquations.appendRow(inequalities[0]);
+  L->m[0].rtyp = coneID; L->m[0].data=(void*) new gfan::ZCone(newInequalities,newEquations);
+  
+  /* these are the cases i=1,...,r-2 */
+  for (int i=1; i<r-1; i++)
+  {
+    newInequalities = inequalities.submatrix(0,0,i-1,c);
+    newInequalities.append(inequalities.submatrix(i+1,0,r,c));
+    newEquations = equations;
+    newEquations.appendRow(inequalities[i]);
+    L->m[i].rtyp = coneID; L->m[i].data=(void*) new gfan::ZCone(newInequalities,newEquations);
+  }
+  
+  /* this is the i=r-1 case */
+  newInequalities = inequalities.submatrix(0,0,r-1,c);
+  newEquations = equations;
+  newEquations.appendRow(inequalities[r]);
+  L->m[r-1].rtyp = coneID; L->m[r-1].data=(void*) new gfan::ZCone(newInequalities,newEquations);
+
+  return L;
+}
+
+BOOLEAN listOfFacets(leftv res, leftv args)
+{
+  leftv u=args;
+  if ((u != NULL) && (u->Typ() == coneID))
+  {
+    gfan::ZCone* zc = (gfan::ZCone*) u->Data();
+    lists L = listOfFacets(*zc);
+    res->rtyp = LIST_CMD;
+    res->data = (void*) L;
+    return FALSE;
+  }
+  WerrorS("listOfFacets: unexpected parameters");
+  return TRUE;
+}
 
 // BOOLEAN grFan(leftv res, leftv h)
 // {
@@ -688,7 +943,7 @@ gfan::ZMatrix rays(gfan::ZFan* zf)
 //         #define USE_ZFAN
 //     #endif
 //     #ifndef USE_ZFAN
-//     res->rtyp=LIST_CMD; //res->rtyp=coneID; res->data(char*)zcone;
+//     res->rtyp=LIST_CMD; //res->rtyp=coneID; res->data(void*)zcone;
 //     res->data=(lists) grfan(I,heuristic,FALSE);
 //     #else
 //     res->rtyp=fanID;
@@ -735,6 +990,7 @@ void bbfan_setup()
   // iiAddCproc("","getCodimension",FALSE,getDimension);
   // iiAddCproc("","getDimension",FALSE,getDimension);
   // iiAddCproc("","getLinealityDimension",FALSE,getLinealityDimension);
+  // iiAddCproc("","isSimplicial",FALSE,isSimplicial);
   /********************************************************/
   iiAddCproc("","isCompatible",FALSE,isCompatible);
   iiAddCproc("","numberOfConesOfDimension",FALSE,numberOfConesOfDimension);
@@ -743,8 +999,11 @@ void bbfan_setup()
   iiAddCproc("","insertCone",FALSE,insertCone);
   iiAddCproc("","removeCone",FALSE,removeCone);
   iiAddCproc("","getCone",FALSE,getCone);
-  iiAddCproc("","isSimplicial",FALSE,isSimplicial);
+  iiAddCproc("","getCones",FALSE,getCones);
   iiAddCproc("","isPure",FALSE,isPure);
+  iiAddCproc("","fanFromString",FALSE,fanFromString);
+  iiAddCproc("","fanViaCones",FALSE,fanViaCones);
+  iiAddCproc("","numberOfConesWithVector",FALSE,numberOfConesWithVector);
   // iiAddCproc("","isComplete",FALSE,isComplete);  not working as expected, should leave this to polymake
   iiAddCproc("","fVector",FALSE,fVector);
   iiAddCproc("","containsInCollection",FALSE,containsInCollection);
