@@ -1,7 +1,9 @@
 #ifndef BBFAN_H
 #define BBFAN_H
 
+#include <Singular/mod2.h>
 #ifdef HAVE_FANS
+
 #include <gfanlib/gfanlib.h>
 
 extern int fanID;
@@ -12,7 +14,7 @@ int getAmbientDimension(gfan::ZFan* zf);
 int getCodimension(gfan::ZFan* zf);
 int getDimension(gfan::ZFan* zf);
 int getLinealityDimension(gfan::ZFan* zf);
-
+int isSimplicial(gfan::ZFan* zf);
 gfan::Matrix<gfan::Integer> rays(gfan::ZFan* zf);
 
 #endif
