@@ -2,6 +2,7 @@
 
 #include <config.h>
 
+#include "timing.h"
 #include "assert.h"
 #include "debug.h"
 
@@ -35,15 +36,15 @@ static CanonicalForm gcd_univar_flintp (const CanonicalForm &, const CanonicalFo
 #endif
 
 static CanonicalForm cf_content ( const CanonicalForm &, const CanonicalForm & );
-static void cf_prepgcd( const CanonicalForm &, const CanonicalForm &, int &, int &, int & );
 
 void out_cf(const char *s1,const CanonicalForm &f,const char *s2);
 
 CanonicalForm chinrem_gcd(const CanonicalForm & FF,const CanonicalForm & GG);
 
 bool
-gcd_test_one ( const CanonicalForm & f, const CanonicalForm & g, bool swap )
+gcd_test_one ( const CanonicalForm & f, const CanonicalForm & g, bool swap, int & d )
 {
+    d= 0;
     int count = 0;
     // assume polys have same level;
 
@@ -111,7 +112,11 @@ gcd_test_one ( const CanonicalForm & f, const CanonicalForm & g, bool swap )
       CanonicalForm primElem, imPrimElem;
       if (p == 2 && d < 6)
       {
-        zz_p::init (p);
+        if (fac_NTL_char != 2)
+        {
+          fac_NTL_char= 2;
+          zz_p::init (p);
+        }
         bool primFail= false;
         Variable vBuf;
         primElem= primitiveElement (v, vBuf, primFail);
@@ -135,7 +140,11 @@ gcd_test_one ( const CanonicalForm & f, const CanonicalForm & g, bool swap )
       }
       else if ((p == 3 && d < 4) || ((p == 5 || p == 7) && d < 3))
       {
-        zz_p::init (p);
+        if (fac_NTL_char != p)
+        {
+          fac_NTL_char= p;
+          zz_p::init (p);
+        }
         bool primFail= false;
         Variable vBuf;
         primElem= primitiveElement (v, vBuf, primFail);
@@ -215,17 +224,11 @@ gcd_test_one ( const CanonicalForm & f, const CanonicalForm & g, bool swap )
       eval2= e (G);
     }
 
-    if ( eval1.taildegree() > 0 && eval2.taildegree() > 0 )
-    {
-        if (passToGF)
-          setCharacteristic (p);
-        if (k > 1)
-          setCharacteristic (p, k, gf_name);
-        return false;
-    }
-
     CanonicalForm c= gcd (eval1, eval2);
-    bool result= c.degree() < 1;
+    d= c.degree();
+    bool result= d < 1;
+    if (d < 0)
+      d= 0;
 
     if (passToGF)
       setCharacteristic (p);
@@ -306,6 +309,28 @@ extgcd ( const CanonicalForm & f, const CanonicalForm & g, CanonicalForm & a, Ca
     return f;
   }
 #ifdef HAVE_NTL
+#ifdef HAVE_FLINT
+  if (( getCharacteristic() > 0 ) && (CFFactory::gettype() != GaloisFieldDomain)
+  &&  (f.level()==g.level()) && isPurePoly(f) && isPurePoly(g))
+  {
+    nmod_poly_t F1, G1, A, B, R;
+    convertFacCF2nmod_poly_t (F1, f);
+    convertFacCF2nmod_poly_t (G1, g);
+    nmod_poly_init (R, getCharacteristic());
+    nmod_poly_init (A, getCharacteristic());
+    nmod_poly_init (B, getCharacteristic());
+    nmod_poly_xgcd (R, A, B, F1, G1);
+    a= convertnmod_poly_t2FacCF (A, f.mvar());
+    b= convertnmod_poly_t2FacCF (B, f.mvar());
+    CanonicalForm r= convertnmod_poly_t2FacCF (R, f.mvar());
+    nmod_poly_clear (F1);
+    nmod_poly_clear (G1);
+    nmod_poly_clear (A);
+    nmod_poly_clear (B);
+    nmod_poly_clear (R);
+    return r;
+  }
+#else
   if (isOn(SW_USE_NTL_GCD_P) && ( getCharacteristic() > 0 ) && (CFFactory::gettype() != GaloisFieldDomain)
   &&  (f.level()==g.level()) && isPurePoly(f) && isPurePoly(g))
   {
@@ -346,6 +371,30 @@ extgcd ( const CanonicalForm & f, const CanonicalForm & g, CanonicalForm & a, Ca
     return convertNTLzzpX2CF(R,f.mvar());
     #endif
   }
+#endif
+#ifdef HAVE_FLINT
+  if (( getCharacteristic() ==0) && (f.level()==g.level())
+       && isPurePoly(f) && isPurePoly(g))
+  {
+    fmpq_poly_t F1, G1;
+    convertFacCF2Fmpq_poly_t (F1, f);
+    convertFacCF2Fmpq_poly_t (G1, g);
+    fmpq_poly_t R, A, B;
+    fmpq_poly_init (R);
+    fmpq_poly_init (A);
+    fmpq_poly_init (B);
+    fmpq_poly_xgcd (R, A, B, F1, G1);
+    a= convertFmpq_poly_t2FacCF (A, f.mvar());
+    b= convertFmpq_poly_t2FacCF (B, f.mvar());
+    CanonicalForm r= convertFmpq_poly_t2FacCF (R, f.mvar());
+    fmpq_poly_clear (F1);
+    fmpq_poly_clear (G1);
+    fmpq_poly_clear (A);
+    fmpq_poly_clear (B);
+    fmpq_poly_clear (R);
+    return r;
+  }
+#else
   if (isOn(SW_USE_NTL_GCD_0) && ( getCharacteristic() ==0)
   && (f.level()==g.level()) && isPurePoly(f) && isPurePoly(g))
   {
@@ -377,11 +426,12 @@ extgcd ( const CanonicalForm & f, const CanonicalForm & g, CanonicalForm & a, Ca
       b=convertNTLZZX2CF(B,f.mvar())*gc;
       CanonicalForm rr=convertZZ2CF(RR);
       ASSERT (!rr.isZero(), "NTL:XGCD failed");
-      r*=rr;
-      if ( r.sign() < 0 ) { r= -r; a= -a; b= -b; }
+      a /= rr;
+      b /= rr;
       return r;
     }
   }
+#endif
 #endif
   // may contain bug in the co-factors, see track 107
   CanonicalForm contf = content( f );
@@ -413,10 +463,10 @@ extgcd ( const CanonicalForm & f, const CanonicalForm & g, CanonicalForm & a, Ca
 }
 //}}}
 
-//{{{ static CanonicalForm balance ( const CanonicalForm & f, const CanonicalForm & q )
+//{{{ static CanonicalForm balance_p ( const CanonicalForm & f, const CanonicalForm & q )
 //{{{ docu
 //
-// balance() - map f from positive to symmetric representation
+// balance_p() - map f from positive to symmetric representation
 //   mod q.
 //
 // This makes sense for univariate polynomials over Z only.
@@ -426,6 +476,29 @@ extgcd ( const CanonicalForm & f, const CanonicalForm & g, CanonicalForm & a, Ca
 //
 //}}}
 static CanonicalForm
+balance_p ( const CanonicalForm & f, const CanonicalForm & q )
+{
+    Variable x = f.mvar();
+    CanonicalForm result = 0, qh = q / 2;
+    CanonicalForm c;
+    CFIterator i;
+    for ( i = f; i.hasTerms(); i++ )
+    {
+        c = i.coeff();
+        if ( c.inCoeffDomain())
+        {
+          if ( c > qh )
+            result += power( x, i.exp() ) * (c - q);
+          else
+            result += power( x, i.exp() ) * c;
+        }
+        else
+          result += power( x, i.exp() ) * balance_p(c,q);
+    }
+    return result;
+}
+
+/*static CanonicalForm
 balance ( const CanonicalForm & f, const CanonicalForm & q )
 {
     Variable x = f.mvar();
@@ -440,7 +513,7 @@ balance ( const CanonicalForm & f, const CanonicalForm & q )
             result += power( x, i.exp() ) * c;
     }
     return result;
-}
+}*/
 //}}}
 
 static CanonicalForm gcd_poly_univar0( const CanonicalForm & F, const CanonicalForm & G, bool primitive )
@@ -516,7 +589,7 @@ static CanonicalForm gcd_poly_univar0( const CanonicalForm & F, const CanonicalF
     if ( i >= 0 )
     {
       // now balance D mod q
-      D = pp( balance( D, q ) );
+      D = pp( balance_p( D, q ) );
       if ( fdivides( D, f ) && fdivides( D, g ) )
         return D * c;
       else
@@ -547,9 +620,10 @@ gcd_poly_p( const CanonicalForm & f, const CanonicalForm & g )
     Ci = content( pi ); Ci1 = content( pi1 );
     pi1 = pi1 / Ci1; pi = pi / Ci;
     C = gcd( Ci, Ci1 );
+    int d= 0;
     if ( !( pi.isUnivariate() && pi1.isUnivariate() ) )
     {
-        if ( gcd_test_one( pi1, pi, true ) )
+        if ( gcd_test_one( pi1, pi, true, d ) )
         {
           C=abs(C);
           //out_cf("GCD:",C,"\n");
@@ -572,12 +646,24 @@ gcd_poly_p( const CanonicalForm & f, const CanonicalForm & g )
     }
     Variable v = f.mvar();
     Hi = power( LC( pi1, v ), delta );
+    int maxNumVars= tmax (getNumVars (pi), getNumVars (pi1));
+
+    if (!(pi.isUnivariate() && pi1.isUnivariate()))
+    {
+      if (size (Hi)*size (pi)/(maxNumVars*3) > 500) //maybe this needs more tuning
+      {
+        On (SW_USE_FF_MOD_GCD);
+        C *= gcd (pi, pi1);
+        Off (SW_USE_FF_MOD_GCD);
+        return C;
+      }
+    }
+
     if ( (delta+1) % 2 )
         bi = 1;
     else
         bi = -1;
-    int maxNumVars= tmax (getNumVars (pi), getNumVars (pi1));
-    CanonicalForm oldPi= pi, oldPi1= pi1;
+    CanonicalForm oldPi= pi, oldPi1= pi1, powHi;
     while ( degree( pi1, v ) > 0 )
     {
         if (!(pi.isUnivariate() && pi1.isUnivariate()))
@@ -594,14 +680,32 @@ gcd_poly_p( const CanonicalForm & f, const CanonicalForm & g )
         pi2 = pi2 / bi;
         pi = pi1; pi1 = pi2;
         maxNumVars= tmax (getNumVars (pi), getNumVars (pi1));
+        if (!(pi1.isUnivariate()) && (size (pi1)/maxNumVars > 500))
+        {
+            On (SW_USE_FF_MOD_GCD);
+            C *= gcd (oldPi, oldPi1);
+            Off (SW_USE_FF_MOD_GCD);
+            return C;
+        }
         if ( degree( pi1, v ) > 0 )
         {
             delta = degree( pi, v ) - degree( pi1, v );
+            powHi= power (Hi, delta-1);
             if ( (delta+1) % 2 )
-                bi = LC( pi, v ) * power( Hi, delta );
+                bi = LC( pi, v ) * powHi*Hi;
             else
-                bi = -LC( pi, v ) * power( Hi, delta );
-            Hi = power( LC( pi1, v ), delta ) / power( Hi, delta-1 );
+                bi = -LC( pi, v ) * powHi*Hi;
+            Hi = power( LC( pi1, v ), delta ) / powHi;
+            if (!(pi.isUnivariate() && pi1.isUnivariate()))
+            {
+              if (size (Hi)*size (pi)/(maxNumVars*3) > 500) //maybe this needs more tuning
+              {
+                On (SW_USE_FF_MOD_GCD);
+                C *= gcd (oldPi, oldPi1);
+                Off (SW_USE_FF_MOD_GCD);
+                return C;
+              }
+            }
         }
     }
     if ( degree( pi1, v ) == 0 )
@@ -636,20 +740,21 @@ gcd_poly_0( const CanonicalForm & f, const CanonicalForm & g )
     Ci = content( pi ); Ci1 = content( pi1 );
     pi1 = pi1 / Ci1; pi = pi / Ci;
     C = gcd( Ci, Ci1 );
+    int d= 0;
     if ( pi.isUnivariate() && pi1.isUnivariate() )
     {
-/*#ifdef HAVE_FLINT
+#ifdef HAVE_FLINT
         if (isPurePoly(pi) && isPurePoly(pi1) )
             return gcd_univar_flint0(pi, pi1 ) * C;
-#else*/
+#else
 #ifdef HAVE_NTL
         if ( isOn(SW_USE_NTL_GCD_0) && isPurePoly(pi) && isPurePoly(pi1) )
             return gcd_univar_ntl0(pi, pi1 ) * C;
 #endif
-//#endif
+#endif
         return gcd_poly_univar0( pi, pi1, true ) * C;
     }
-    else if ( gcd_test_one( pi1, pi, true ) )
+    else if ( gcd_test_one( pi1, pi, true, d ) )
       return C;
     Variable v = f.mvar();
     Hi = power( LC( pi1, v ), delta );
@@ -696,27 +801,9 @@ int si_factor_reminder=1;
 CanonicalForm gcd_poly ( const CanonicalForm & f, const CanonicalForm & g )
 {
   CanonicalForm fc, gc, d1;
-  int mp, cc, p1, pe;
-  mp = f.level()+1;
   bool fc_isUnivariate=f.isUnivariate();
   bool gc_isUnivariate=g.isUnivariate();
   bool fc_and_gc_Univariate=fc_isUnivariate && gc_isUnivariate;
-  cf_prepgcd( f, g, cc, p1, pe);
-  if ( cc != 0 )
-  {
-    if ( cc > 0 )
-    {
-      fc = replacevar( f, Variable(cc), Variable(mp) );
-      gc = g;
-    }
-    else
-    {
-      fc = replacevar( g, Variable(-cc), Variable(mp) );
-      gc = f;
-    }
-    return cf_content( fc, gc );
-  }
-// now each appearing variable is in f and g
   fc = f;
   gc = g;
   if ( getCharacteristic() != 0 )
@@ -738,54 +825,14 @@ CanonicalForm gcd_poly ( const CanonicalForm & f, const CanonicalForm & g )
     }
     else
     #endif
-    if ( p1 == fc.level() )
-      fc = gcd_poly_p( fc, gc );
-    else
-    {
-      fc = replacevar( fc, Variable(p1), Variable(mp) );
-      gc = replacevar( gc, Variable(p1), Variable(mp) );
-      fc = replacevar( gcd_poly_p( fc, gc ), Variable(mp), Variable(p1) );
-    }
+    fc = gcd_poly_p( fc, gc );
   }
   else if (!fc_and_gc_Univariate)
   {
     if ( isOn( SW_USE_EZGCD ) )
-    {
       fc= ezgcd (fc, gc);
-      /*if ( pe == 1 )
-        fc = ezgcd( fc, gc );
-      else if ( pe > 0 )// no variable at position 1
-      {
-        fc = replacevar( fc, Variable(pe), Variable(1) );
-        gc = replacevar( gc, Variable(pe), Variable(1) );
-        fc = replacevar( ezgcd( fc, gc ), Variable(1), Variable(pe) );
-      }
-      else
-      {
-        pe = -pe;
-        fc = swapvar( fc, Variable(pe), Variable(1) );
-        gc = swapvar( gc, Variable(pe), Variable(1) );
-        fc = swapvar( ezgcd( fc, gc ), Variable(1), Variable(pe) );
-      }*/
-    }
-    else if (
-    isOn(SW_USE_CHINREM_GCD)
-    && (isPurePoly_m(fc)) && (isPurePoly_m(gc))
-    )
-    {
-    #if 0
-      if ( p1 == fc.level() )
-        fc = chinrem_gcd( fc, gc );
-      else
-      {
-        fc = replacevar( fc, Variable(p1), Variable(mp) );
-        gc = replacevar( gc, Variable(p1), Variable(mp) );
-        fc = replacevar( chinrem_gcd( fc, gc ), Variable(mp), Variable(p1) );
-      }
-    #else
+    else if (isOn(SW_USE_CHINREM_GCD))
       fc = chinrem_gcd( fc, gc);
-    #endif
-    }
     else
     {
        fc = gcd_poly_0( fc, gc );
@@ -1099,7 +1146,7 @@ gcd_univar_flint0( const CanonicalForm & F, const CanonicalForm & G )
 *    pe is used in "ezgcd" and
 *    p1 in "gcd_poly1"
 */
-static
+/*static
 void optvalues ( const int * df, const int * dg, const int n, int & p1, int &pe )
 {
     int i, o1, oe;
@@ -1129,12 +1176,12 @@ void optvalues ( const int * df, const int * dg, const int n, int & p1, int &pe
         }
         i--;
     }
-}
+}*/
 
 /*
 *  make some changes of variables, see optvalues
 */
-static void
+/*static void
 cf_prepgcd( const CanonicalForm & f, const CanonicalForm & g, int & cc, int & p1, int &pe )
 {
     int i, k, n;
@@ -1181,31 +1228,11 @@ cf_prepgcd( const CanonicalForm & f, const CanonicalForm & g, int & cc, int & p1
 
     delete [] degsf;
     delete [] degsg;
-}
+}*/
 
-
-static CanonicalForm
-balance_p ( const CanonicalForm & f, const CanonicalForm & q )
-{
-    Variable x = f.mvar();
-    CanonicalForm result = 0, qh = q / 2;
-    CanonicalForm c;
-    CFIterator i;
-    for ( i = f; i.hasTerms(); i++ )
-    {
-        c = i.coeff();
-        if ( c.inCoeffDomain())
-        {
-          if ( c > qh )
-            result += power( x, i.exp() ) * (c - q);
-          else
-            result += power( x, i.exp() ) * c;
-        }
-        else
-          result += power( x, i.exp() ) * balance_p(c,q);
-    }
-    return result;
-}
+TIMING_DEFINE_PRINT(chinrem_termination)
+TIMING_DEFINE_PRINT(chinrem_recursion)
+TIMING_DEFINE_PRINT(chinrem_reconstruction)
 
 CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
 {
@@ -1274,6 +1301,7 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
     setCharacteristic( p );
     fp= mapinto (f);
     gp= mapinto (g);
+    TIMING_START (chinrem_recursion)
 #ifdef HAVE_NTL
     if (size (fp)/maxNumVars > 500 && size (gp)/maxNumVars > 500)
       Dp = GCD_small_p (fp, gp, cofp, cogp);
@@ -1288,6 +1316,8 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
     cofp= fp/Dp;
     cogp= gp/Dp;
 #endif
+    TIMING_END_AND_PRINT (chinrem_recursion,
+                          "time for gcd mod p in modular gcd: ");
     Dp /=Dp.lc();
     cofp /= lc (cofp);
     cogp /= lc (cogp);
@@ -1338,9 +1368,12 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
     }
     if ( i >= 0 )
     {
+      TIMING_START (chinrem_reconstruction);
       Dn= Farey(D,q);
       cofn= Farey(cof,q);
       cogn= Farey(cog,q);
+      TIMING_END_AND_PRINT (chinrem_reconstruction,
+                           "time for rational reconstruction in modular gcd: ");
       int is_rat= isOn (SW_RATIONAL);
       On (SW_RATIONAL);
       cd = bCommonDen( Dn ); // we need On(SW_RATIONAL)
@@ -1354,12 +1387,17 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
       else
         equal= true;
       //Dn /=vcontent(Dn,Variable(1));
+      TIMING_START (chinrem_termination);
       if ((terminationTest (f,g, cofn, cogn, Dn)) ||
           ((equal || q > b) && fdivides (Dn, f) && fdivides (Dn, g)))
       {
+        TIMING_END_AND_PRINT (chinrem_termination,
+                            "time for successful termination in modular gcd: ");
         //printf(" -> success\n");
         return Dn*gcdcfcg;
       }
+      TIMING_END_AND_PRINT (chinrem_termination,
+                          "time for unsuccessful termination in modular gcd: ");
       equal= false;
       //else: try more primes
     }
