@@ -26,6 +26,8 @@
 TIMING_DEFINE_PRINT(fac_uni_factorizer)
 TIMING_DEFINE_PRINT(fac_bi_hensel_lift)
 TIMING_DEFINE_PRINT(fac_bi_factor_recombination)
+TIMING_DEFINE_PRINT(fac_bi_evaluation)
+TIMING_DEFINE_PRINT(fac_bi_shift_to_zero)
 
 // bound on coeffs of f (cf. Musser: Multivariate Polynomial Factorization,
 //                          Gelfond: Transcendental and Algebraic Numbers)
@@ -98,22 +100,15 @@ coeffBound ( const CanonicalForm & f, int p, const CanonicalForm& mipo )
         M += degs[i];
         K *= degs[i] + 1;
     }
-    K /= power (CanonicalForm (2), k);
-    K= K.sqrt()+1;
+    K /= power (CanonicalForm (2), k/2);
     K *= power (CanonicalForm (2), M);
     int N= degree (mipo);
     CanonicalForm b;
     b= 2*power (maxNorm (f), N)*power (maxNorm (mipo), 4*N)*K*
-       power (CanonicalForm (2), N)*(CanonicalForm (M+1).sqrt()+1)*
-       power (CanonicalForm (N+1).sqrt()+1, 7*N);
+       power (CanonicalForm (2), N)*
+       power (CanonicalForm (N+1), 4*N);
     b /= power (abs (lc (mipo)), N);
 
-    ZZX NTLmipo= convertFacCF2NTLZZX (mipo);
-    ZZX NTLLcf= convertFacCF2NTLZZX (Lc (f));
-    ZZ NTLf= resultant (NTLmipo, NTLLcf);
-    ZZ NTLD= discriminant (NTLmipo);
-    b /= abs (convertZZ2CF (NTLf))*abs (convertZZ2CF (NTLD));
-
     CanonicalForm B = p;
     k = 1;
     while ( B < b ) {
@@ -184,171 +179,6 @@ CanonicalForm evalPoint (const CanonicalForm& F, int& i)
   } while (1);
 }
 
-CFList
-earlyFactorDetection0 (CanonicalForm& F, CFList& factors,int& adaptedLiftBound,
-                      DegreePattern& degs, bool& success, int deg)
-{
-  DegreePattern bufDegs1= degs;
-  DegreePattern bufDegs2;
-  CFList result;
-  CFList T= factors;
-  CanonicalForm buf= F;
-  CanonicalForm LCBuf= LC (buf, Variable (1));
-  CanonicalForm g, quot;
-  CanonicalForm M= power (F.mvar(), deg);
-  adaptedLiftBound= 0;
-  int d= degree (F) + degree (LCBuf, F.mvar());
-  for (CFListIterator i= factors; i.hasItem(); i++)
-  {
-    if (!bufDegs1.find (degree (i.getItem(), 1)))
-      continue;
-    else
-    {
-      g= i.getItem() (0, 1);
-      g *= LCBuf;
-      g= mod (g, M);
-      if (fdivides (LC (g), LCBuf))
-      {
-        g= mulMod2 (i.getItem(), LCBuf, M);
-        g /= content (g, Variable (1));
-        if (fdivides (g, buf, quot))
-        {
-          result.append (g);
-          buf= quot;
-          d -= degree (g) + degree (LC (g, Variable (1)), F.mvar());
-          LCBuf= LC (buf, Variable (1));
-          T= Difference (T, CFList (i.getItem()));
-
-          // compute new possible degree pattern
-          bufDegs2= DegreePattern (T);
-          bufDegs1.intersect (bufDegs2);
-          bufDegs1.refine ();
-          if (bufDegs1.getLength() <= 1)
-          {
-            result.append (buf);
-            break;
-          }
-        }
-      }
-    }
-  }
-  adaptedLiftBound= d + 1;
-  if (d < deg)
-  {
-    factors= T;
-    degs= bufDegs1;
-    F= buf;
-    success= true;
-  }
-  if (bufDegs1.getLength() <= 1)
-    degs= bufDegs1;
-  return result;
-}
-
-
-CFList
-henselLiftAndEarly0 (CanonicalForm& A, bool& earlySuccess, CFList&
-                    earlyFactors, DegreePattern& degs, int& liftBound,
-                    const CFList& uniFactors, const CanonicalForm& eval)
-{
-  int sizeOfLiftPre;
-  int * liftPre= getLiftPrecisions (A, sizeOfLiftPre, degree (LC (A, 1), 2));
-
-  Variable x= Variable (1);
-  Variable y= Variable (2);
-  CFArray Pi;
-  CFList diophant;
-  CFList bufUniFactors= uniFactors;
-  bufUniFactors.insert (LC (A, x));
-  CFMatrix M= CFMatrix (liftBound, bufUniFactors.length() - 1);
-  earlySuccess= false;
-  int newLiftBound= 0;
-  int smallFactorDeg= tmin (11, liftPre [sizeOfLiftPre- 1] + 1);//this is a tunable parameter
-  if (smallFactorDeg >= liftBound || degree (A,y) <= 4)
-    henselLift12 (A, bufUniFactors, liftBound, Pi, diophant, M);
-  else if (sizeOfLiftPre > 1 && sizeOfLiftPre < 30)
-  {
-    henselLift12 (A, bufUniFactors, smallFactorDeg, Pi, diophant, M);
-    earlyFactors= earlyFactorDetection0 (A, bufUniFactors, newLiftBound,
-                                        degs, earlySuccess,
-                                        smallFactorDeg);
-    if (degs.getLength() > 1 && !earlySuccess &&
-        smallFactorDeg != liftPre [sizeOfLiftPre-1] + 1)
-    {
-      if (newLiftBound > liftPre[sizeOfLiftPre-1]+1)
-      {
-        bufUniFactors.insert (LC (A, x));
-        henselLiftResume12 (A, bufUniFactors, smallFactorDeg,
-                            liftPre[sizeOfLiftPre-1] + 1, Pi, diophant, M);
-        earlyFactors= earlyFactorDetection0 (A, bufUniFactors, newLiftBound,
-                      degs, earlySuccess, liftPre[sizeOfLiftPre-1] + 1);
-      }
-    }
-    else if (earlySuccess)
-      liftBound= newLiftBound;
-    int i= sizeOfLiftPre - 1;
-    while (degs.getLength() > 1 && !earlySuccess && i - 1 >= 0)
-    {
-      if (newLiftBound > liftPre[i] + 1)
-      {
-        bufUniFactors.insert (LC (A, x));
-        henselLiftResume12 (A, bufUniFactors, liftPre[i] + 1,
-                            liftPre[i-1] + 1, Pi, diophant, M);
-        earlyFactors= earlyFactorDetection0 (A, bufUniFactors, newLiftBound,
-                      degs, earlySuccess, liftPre[i-1] + 1);
-      }
-      else
-      {
-        liftBound= newLiftBound;
-        break;
-      }
-      i--;
-    }
-    if (earlySuccess)
-      liftBound= newLiftBound;
-    //after here all factors are lifted to liftPre[sizeOfLiftPre-1]
-  }
-  else
-  {
-    henselLift12 (A, bufUniFactors, smallFactorDeg, Pi, diophant, M);
-    earlyFactors= earlyFactorDetection0 (A, bufUniFactors, newLiftBound,
-                                        degs, earlySuccess,
-                                        smallFactorDeg);
-    int i= 1;
-    while ((degree (A,y)/4)*i + 4 <= smallFactorDeg)
-      i++;
-    if (degs.getLength() > 1 && !earlySuccess)
-    {
-      bufUniFactors.insert (LC (A, x));
-      henselLiftResume12 (A, bufUniFactors, smallFactorDeg,
-                          (degree (A, y)/4)*i + 4, Pi, diophant, M);
-      earlyFactors= earlyFactorDetection0 (A, bufUniFactors, newLiftBound,
-                    degs, earlySuccess, (degree (A, y)/4)*i + 4);
-    }
-    while (degs.getLength() > 1 && !earlySuccess && i < 4)
-    {
-      if (newLiftBound > (degree (A, y)/4)*i + 4)
-      {
-        bufUniFactors.insert (LC (A, x));
-        henselLiftResume12 (A, bufUniFactors, (degree (A,y)/4)*i + 4,
-                            (degree (A, y)/4)*(i+1) + 4, Pi, diophant, M);
-        earlyFactors= earlyFactorDetection0 (A, bufUniFactors, newLiftBound,
-                      degs, earlySuccess, (degree (A, y)/4)*(i+1) + 4);
-      }
-      else
-      {
-        liftBound= newLiftBound;
-        break;
-      }
-      i++;
-    }
-    if (earlySuccess)
-      liftBound= newLiftBound;
-  }
-
-  return bufUniFactors;
-}
-
 CFList biFactorize (const CanonicalForm& F, const Variable& v)
 {
   if (F.inCoeffDomain())
@@ -497,38 +327,55 @@ CFList biFactorize (const CanonicalForm& F, const Variable& v)
   for (int i= 0; i < factorNums; i++)
   {
     bufAeval= A;
+    TIMING_START (fac_bi_evaluation);
     bufAeval= evalPoint (A, bufEvaluation);
+    TIMING_END_AND_PRINT (fac_bi_evaluation, "time for eval point over Q: ");
 
     bufAeval2= buf;
+    TIMING_START (fac_bi_evaluation);
     bufAeval2= evalPoint (buf, bufEvaluation2);
+    TIMING_END_AND_PRINT (fac_bi_evaluation,
+                          "time for eval point over Q in y: ");
 
     // univariate factorization
     TIMING_START (fac_uni_factorizer);
-
     if (extension)
       bufUniFactors= conv (factorize (bufAeval, v));
     else
       bufUniFactors= conv (factorize (bufAeval, true));
     TIMING_END_AND_PRINT (fac_uni_factorizer,
-                          "time for univariate factorization: ");
+                          "time for univariate factorization over Q: ");
     DEBOUTLN (cerr, "prod (bufUniFactors)== bufAeval " <<
               (prod (bufUniFactors) == bufAeval));
 
+    if (bufUniFactors.getFirst().inCoeffDomain())
+      bufUniFactors.removeFirst();
+
+    if (bufUniFactors.length() == 1)
+    {
+      factors.append (A);
+
+      appendSwapDecompress (factors, conv (contentAxFactors),
+                            conv (contentAyFactors), swap, swap2, N);
+
+      if (isOn (SW_RATIONAL))
+        normalize (factors);
+      return factors;
+    }
+
     TIMING_START (fac_uni_factorizer);
     if (extension)
       bufUniFactors2= conv (factorize (bufAeval2, v));
     else
       bufUniFactors2= conv (factorize (bufAeval2, true));
     TIMING_END_AND_PRINT (fac_uni_factorizer,
-                          "time for univariate factorization in y: ");
+                          "time for univariate factorization in y over Q: ");
     DEBOUTLN (cerr, "prod (bufuniFactors2)== bufAeval2 " <<
               (prod (bufUniFactors2) == bufAeval2));
 
-    if (bufUniFactors.getFirst().inCoeffDomain())
-      bufUniFactors.removeFirst();
     if (bufUniFactors2.getFirst().inCoeffDomain())
       bufUniFactors2.removeFirst();
-    if (bufUniFactors.length() == 1 || bufUniFactors2.length() == 1)
+    if (bufUniFactors2.length() == 1)
     {
       factors.append (A);
 
@@ -643,12 +490,14 @@ CFList biFactorize (const CanonicalForm& F, const Variable& v)
     return factors;
   }
 
+  A *= bCommonDen (A);
   A= A (y + evaluation, y);
 
   int liftBound= degree (A, y) + 1;
 
   modpk b= modpk();
   bool mipoHasDen= false;
+  CanonicalForm den= 1;
   if (!extension)
   {
     Off (SW_RATIONAL);
@@ -669,6 +518,14 @@ CFList biFactorize (const CanonicalForm& F, const Variable& v)
   else
   {
     A /= Lc (Aeval);
+    mipoHasDen= !bCommonDen(mipo).isOne();
+    mipo *= bCommonDen (mipo);
+    ZZX NTLmipo= convertFacCF2NTLZZX (mipo);
+    ZZX NTLLcf= convertFacCF2NTLZZX (Lc (A*bCommonDen (A)));
+    ZZ NTLf= resultant (NTLmipo, NTLLcf);
+    ZZ NTLD= discriminant (NTLmipo);
+    den= abs (convertZZ2CF (NTLD*NTLf));
+
     // make factors elements of Z(a)[x] disable for modularDiophant
     CanonicalForm multiplier= 1;
     for (CFListIterator i= uniFactors; i.hasItem(); i++)
@@ -679,12 +536,9 @@ CFList biFactorize (const CanonicalForm& F, const Variable& v)
     A *= multiplier;
     A *= bCommonDen (A);
 
-    mipoHasDen= !bCommonDen(mipo).isOne();
-    mipo *= bCommonDen (mipo);
     Off (SW_RATIONAL);
     int i= 0;
-    ZZX NTLmipo= convertFacCF2NTLZZX (mipo);
-    CanonicalForm discMipo= convertZZ2CF (discriminant (NTLmipo));
+    CanonicalForm discMipo= convertZZ2CF (NTLD);
     findGoodPrime (F*discMipo,i);
     findGoodPrime (Aeval*discMipo,i);
     findGoodPrime (A*discMipo,i);
@@ -705,8 +559,9 @@ CFList biFactorize (const CanonicalForm& F, const Variable& v)
   TIMING_START (fac_bi_hensel_lift);
   uniFactors= henselLiftAndEarly
               (A, earlySuccess, earlyFactors, degs, liftBound,
-               uniFactors, dummy, evaluation, b);
-  TIMING_END_AND_PRINT (fac_bi_hensel_lift, "time for hensel lifting: ");
+               uniFactors, dummy, evaluation, b, den);
+  TIMING_END_AND_PRINT (fac_bi_hensel_lift,
+                        "time for bivariate hensel lifting over Q: ");
   DEBOUTLN (cerr, "lifted factors= " << uniFactors);
 
   CanonicalForm MODl= power (y, liftBound);
@@ -721,8 +576,15 @@ CFList biFactorize (const CanonicalForm& F, const Variable& v)
       iter.getItem()= replacevar (iter.getItem(), vv, v);
   }
 
+  On (SW_RATIONAL);
+  A *= bCommonDen (A);
+  Off (SW_RATIONAL);
+
+  TIMING_START (fac_bi_factor_recombination);
   factors= factorRecombination (uniFactors, A, MODl, degs, 1,
-                                uniFactors.length()/2, b);
+                                uniFactors.length()/2, b, den);
+  TIMING_END_AND_PRINT (fac_bi_factor_recombination,
+                        "time for bivariate factor recombination over Q: ");
 
   On (SW_RATIONAL);
 
