@@ -16,6 +16,7 @@
 
 #include <config.h>
 
+#include "timing.h"
 #include "assert.h"
 
 #include "facFqBivarUtil.h"
@@ -27,6 +28,9 @@
 #include "cfNewtonPolygon.h"
 #include "fac_util.h"
 
+TIMING_DEFINE_PRINT(fac_fq_bi_sqrf)
+TIMING_DEFINE_PRINT(fac_fq_bi_factor_sqrf)
+
 static const double log2exp= 1.442695041;
 
 /// Factorization of a squarefree bivariate polynomials over an arbitrary finite
@@ -273,13 +277,19 @@ FpBiFactorize (const CanonicalForm & G, ///< [in] a bivariate poly
   vec_ZZ S;
   F= compress (F, M, S);
 
+  TIMING_START (fac_fq_bi_sqrf);
   CFFList sqrf= FpSqrf (F, false);
+  TIMING_END_AND_PRINT (fac_fq_bi_sqrf,
+                       "time for bivariate sqrf factors over Fp: ");
   CFList bufResult;
   sqrf.removeFirst();
   CFListIterator i;
   for (CFFListIterator iter= sqrf; iter.hasItem(); iter++)
   {
+    TIMING_START (fac_fq_bi_factor_sqrf);
     bufResult= biFactorize (iter.getItem().factor(), info);
+    TIMING_END_AND_PRINT (fac_fq_bi_factor_sqrf,
+                          "time to factor bivariate sqrf factors over Fp: ");
     for (i= bufResult; i.hasItem(); i++)
       result.append (CFFactor (N (decompress (i.getItem(), M, S)),
                                iter.getItem().exp()));
@@ -374,13 +384,19 @@ FqBiFactorize (const CanonicalForm & G, ///< [in] a bivariate poly
   vec_ZZ S;
   F= compress (F, M, S);
 
+  TIMING_START (fac_fq_bi_sqrf);
   CFFList sqrf= FqSqrf (F, alpha, false);
+  TIMING_END_AND_PRINT (fac_fq_bi_sqrf,
+                       "time for bivariate sqrf factors over Fq: ");
   CFList bufResult;
   sqrf.removeFirst();
   CFListIterator i;
   for (CFFListIterator iter= sqrf; iter.hasItem(); iter++)
   {
+    TIMING_START (fac_fq_bi_factor_sqrf);
     bufResult= biFactorize (iter.getItem().factor(), info);
+    TIMING_END_AND_PRINT (fac_fq_bi_factor_sqrf,
+                          "time to factor bivariate sqrf factors over Fq: ");
     for (i= bufResult; i.hasItem(); i++)
       result.append (CFFactor (N (decompress (i.getItem(), M, S)),
                                iter.getItem().exp()));
@@ -476,13 +492,19 @@ GFBiFactorize (const CanonicalForm & G, ///< [in] a bivariate poly
   vec_ZZ S;
   F= compress (F, M, S);
 
+  TIMING_START (fac_fq_bi_sqrf);
   CFFList sqrf= GFSqrf (F, false);
+  TIMING_END_AND_PRINT (fac_fq_bi_sqrf,
+                       "time for bivariate sqrf factors over GF: ");
   CFList bufResult;
   sqrf.removeFirst();
   CFListIterator i;
   for (CFFListIterator iter= sqrf; iter.hasItem(); iter++)
   {
+    TIMING_START (fac_fq_bi_factor_sqrf);
     bufResult= biFactorize (iter.getItem().factor(), info);
+    TIMING_END_AND_PRINT (fac_fq_bi_factor_sqrf,
+                          "time to factor bivariate sqrf factors over GF: ");
     for (i= bufResult; i.hasItem(); i++)
       result.append (CFFactor (N (decompress (i.getItem(), M, S)),
                                iter.getItem().exp()));
@@ -565,18 +587,19 @@ extFactorRecombination (
 /// @sa extFactorRecombination(), earlyFactorDetectection()
 CFList
 factorRecombination (
-                CFList& factors,       ///< [in,out] list of lifted factors
-                                       ///< that are monic wrt Variable (1)
-                CanonicalForm& F,      ///< [in,out] poly to be factored
-                const CanonicalForm& M,///< [in] Variable (2)^liftBound
-                DegreePattern& degs,   ///< [in] degree pattern
-                int s,                 ///< [in] algorithm starts checking
-                                       ///< subsets of size s
-                int thres,             ///< [in] threshold for the size of
-                                       ///< subsets which are checked, for a
-                                       ///< full factor recombination choose
-                                       ///< thres= factors.length()/2
-                const modpk& b=modpk() ///< [in] coeff bound
+            CFList& factors,            ///< [in,out] list of lifted factors
+                                        ///< that are monic wrt Variable (1)
+            CanonicalForm& F,           ///< [in,out] poly to be factored
+            const CanonicalForm& M,     ///< [in] Variable (2)^liftBound
+            DegreePattern& degs,        ///< [in] degree pattern
+            int s,                      ///< [in] algorithm starts checking
+                                        ///< subsets of size s
+            int thres,                  ///< [in] threshold for the size of
+                                        ///< subsets which are checked, for a
+                                        ///< full factor recombination choose
+                                        ///< thres= factors.length()/2
+            const modpk& b=modpk(),     ///< [in] coeff bound
+            const CanonicalForm& den= 1 ///< [in] bound on the den if over Q (a)
                     );
 
 /// chooses a field extension.
@@ -670,7 +693,8 @@ henselLiftAndEarly (
         const CFList& uniFactors,  ///< [in] univariate factors
         const ExtensionInfo& info, ///< [in] information about extension
         const CanonicalForm& eval, ///< [in] evaluation point
-        modpk& b                   ///< [in] coeff bound
+        modpk& b,                  ///< [in] coeff bound
+        CanonicalForm& den         ///< [in] bound on the den if over Q(a)
                   );
 
 /// hensel Lifting and early factor detection
