@@ -15,6 +15,7 @@
 #define FAC_FQ_FACTORIZE_H
 
 #include <config.h>
+#include "timing.h"
 
 #include "facFqBivar.h"
 #include "DegreePattern.h"
@@ -25,6 +26,9 @@
 
 #ifdef HAVE_NTL
 
+TIMING_DEFINE_PRINT (fac_fq_squarefree)
+TIMING_DEFINE_PRINT (fac_fq_factor_squarefree)
+
 /// Factorization over a finite field
 ///
 /// @return @a multiFactorize returns a factorization of F
@@ -150,14 +154,20 @@ CFFList FpFactorize (const CanonicalForm& G,///< [in] a multivariate poly
   ExtensionInfo info= ExtensionInfo (false);
   Variable a= Variable (1);
   CanonicalForm LcF= Lc (F);
+  TIMING_START (fac_fq_squarefree);
   CFFList sqrf= FpSqrf (F, false);
+  TIMING_END_AND_PRINT (fac_fq_squarefree,
+                        "time for squarefree factorization over Fq: ");
   CFFList result;
   CFList bufResult;
   sqrf.removeFirst();
   CFListIterator i;
   for (CFFListIterator iter= sqrf; iter.hasItem(); iter++)
   {
+    TIMING_START (fac_fq_factor_squarefree);
     bufResult= multiFactorize (iter.getItem().factor(), info);
+    TIMING_END_AND_PRINT (fac_fq_factor_squarefree,
+                          "time to factorize sqrfree factor over Fq: ");
     for (i= bufResult; i.hasItem(); i++)
       result.append (CFFactor (i.getItem(), iter.getItem().exp()));
   }
@@ -227,14 +237,20 @@ CFFList FqFactorize (const CanonicalForm& G, ///< [in] a multivariate poly
 
   ExtensionInfo info= ExtensionInfo (alpha, false);
   CanonicalForm LcF= Lc (F);
+  TIMING_START (fac_fq_squarefree);
   CFFList sqrf= FqSqrf (F, alpha, false);
+  TIMING_END_AND_PRINT (fac_fq_squarefree,
+                        "time for squarefree factorization over Fq: ");
   CFFList result;
   CFList bufResult;
   sqrf.removeFirst();
   CFListIterator i;
   for (CFFListIterator iter= sqrf; iter.hasItem(); iter++)
   {
+    TIMING_START (fac_fq_factor_squarefree);
     bufResult= multiFactorize (iter.getItem().factor(), info);
+    TIMING_END_AND_PRINT (fac_fq_factor_squarefree,
+                          "time to factorize sqrfree factor over Fq: ");
     for (i= bufResult; i.hasItem(); i++)
       result.append (CFFactor (i.getItem(), iter.getItem().exp()));
   }
@@ -545,31 +561,6 @@ evaluationWRTDifferentSecondVars (
                     const CanonicalForm& A   ///<[in] some poly
                                  );
 
-/// evaluate F successively n-2 at 0
-///
-/// @return returns a list of successive evaluations of F, ending with F
-CFList evaluateAtZero (const CanonicalForm& F ///< [in] some poly
-                      );
-
-/// divides factors by their content wrt. Variable(1) and checks if these polys
-/// divide F
-///
-/// @return returns factors of F
-CFList recoverFactors (const CanonicalForm& F, ///< [in] some poly F
-                       const CFList& factors   ///< [in] some list of
-                                               ///< factor candidates
-                      );
-
-/// divides factors shifted by evaluation by their content wrt. Variable(1) and 
-/// checks if these polys divide F
-///
-/// @return returns factors of F
-CFList recoverFactors (const CanonicalForm& F,  ///< [in] some poly F
-                       const CFList& factors,   ///< [in] some list of
-                                                ///< factor candidates
-                       const CFList& evaluation
-                      );
-
 /// refine a bivariate factorization of A with l factors to one with
 /// minFactorsLength
 void
@@ -593,15 +584,23 @@ buildUniFactors (const CFList& biFactors,       ///< [in] a list of polys
                  const Variable& y              ///< [in] some variable
                 );
 
+
+/// sort bivariate factors in Aeval such that their corresponding univariate
+/// factors coincide with uniFactors
+void sortByUniFactors (CFList*& Aeval,          ///< [in,out] array of bivariate
+                                                ///< factors
+                       int AevalLength,         ///< [in] length of Aeval
+                       const CFList& uniFactors,///< [in] univariate factors
+                       const CFList& evaluation ///< [in] evaluation point
+                      );
+
 /// extract leading coefficients wrt Variable(1) from bivariate factors obtained
 /// from factorizations of A wrt different second variables
 void
 getLeadingCoeffs (const CanonicalForm& A,  ///< [in] some poly
-                  CFList*& Aeval,          ///< [in,out] array of bivariate
+                  CFList*& Aeval           ///< [in,out] array of bivariate
                                            ///< factors, returns the leading
                                            ///< coefficients of these factors
-                  const CFList& uniFactors,///< [in] univariate factors of A
-                  const CFList& evaluation ///< [in] evaluation point
                  );
 
 /// normalize precomputed leading coefficients such that leading coefficients
@@ -653,24 +652,34 @@ gcdFreeBasis (CFFList& factors1, ///< [in,out] list of factors, returns gcd free
                                  ///< factors
              );
 
-/// evaluate @a F at @a evaluation
+/// computes a list l of length length(LCFFactors)+1 of polynomials such that
+/// prod (l)=LCF, note that the first entry of l may be non constant. Intended
+/// to be used to precompute coefficients of a polynomial f from its bivariate 
+/// factorizations.
 ///
-/// @return @a evaluateAtEval returns a list containing the successive 
-/// evaluations of @a F, last entry is @a F again
+/// @return see above
 CFList
-evaluateAtEval (const CanonicalForm& F,   ///<[in] some poly
-                const CFArray& evaluation ///<[in] some evaluation point
-               );
-
-/// evaluate @a F at @a evaluation
-///
-/// @return @a evaluateAtEval returns a list containing the successive
-/// evaluations of @a F starting at level @a l, last entry is @a F again
-CFList
-evaluateAtEval (const CanonicalForm& F,  ///<[in] some poly
-                const CFList& evaluation,///<[in] some evaluation point
-                int l                    ///<[in] level to start at
-               );
+precomputeLeadingCoeff (const CanonicalForm& LCF,       ///<[in] a multivariate
+                                                        ///< poly
+                        const CFList& LCFFactors,       ///<[in] a list of
+                                                        ///< univariate factors
+                                                        ///< of LCF of level 2
+                        const Variable& alpha,          ///<[in] algebraic var.
+                        const CFList& evaluation,       ///<[in] an evaluation
+                                                        ///< point having
+                                                        ///< lSecondVarLCs+1
+                                                        ///< components
+                        CFList* & differentSecondVarLCs,///<[in] LCs of factors
+                                                        ///< of f wrt different
+                                                        ///< second variables
+                        int lSecondVarLCs,              ///<[in] length of the
+                                                        ///< above
+                        Variable& y                     ///<[in,out] if y.level()
+                                                        ///< is not 1 on output
+                                                        ///< the second variable
+                                                        ///< has been changed to
+                                                        ///< y
+                       );
 
 #endif
 
