@@ -27,6 +27,7 @@
 #include "cf_algorithm.h"
 #include "cf_primes.h"
 #include "facBivar.h"
+#include "facNTLzzpEXGCD.h"
 
 #ifdef HAVE_NTL
 #include <NTL/lzz_pEX.h>
@@ -35,7 +36,11 @@
 static
 CFList productsNTL (const CFList& factors, const CanonicalForm& M)
 {
-  zz_p::init (getCharacteristic());
+  if (fac_NTL_char != getCharacteristic())
+  {
+    fac_NTL_char= getCharacteristic();
+    zz_p::init (getCharacteristic());
+  }
   zz_pX NTLMipo= convertFacCF2NTLzzpX (M);
   zz_pE::init (NTLMipo);
   zz_pEX prod;
@@ -97,19 +102,47 @@ void tryDiophantine (CFList& result, const CanonicalForm& F,
     i++;
   buf1= bufFactors.getFirst();
   buf2= i.getItem();
+#ifdef HAVE_NTL
+  Variable x= Variable (1);
+  if (fac_NTL_char != getCharacteristic())
+  {
+    fac_NTL_char= getCharacteristic();
+    zz_p::init (getCharacteristic());
+  }
+  zz_pX NTLMipo= convertFacCF2NTLzzpX (M);
+  zz_pE::init (NTLMipo);
+  zz_pEX NTLbuf1, NTLbuf2, NTLbuf3, NTLS, NTLT;
+  NTLbuf1= convertFacCF2NTLzz_pEX (buf1, NTLMipo);
+  NTLbuf2= convertFacCF2NTLzz_pEX (buf2, NTLMipo);
+  tryNTLXGCD (NTLbuf3, NTLS, NTLT, NTLbuf1, NTLbuf2, fail);
+  if (fail)
+    return;
+  S= convertNTLzz_pEX2CF (NTLS, x, M.mvar());
+  T= convertNTLzz_pEX2CF (NTLT, x, M.mvar());
+#else
   tryExtgcd (buf1, buf2, M, buf3, S, T, fail);
   if (fail)
     return;
+#endif
   result.append (S);
   result.append (T);
   if (i.hasItem())
     i++;
   for (; i.hasItem(); i++)
   {
+#ifdef HAVE_NTL
+    NTLbuf1= convertFacCF2NTLzz_pEX (i.getItem(), NTLMipo);
+    tryNTLXGCD (NTLbuf3, NTLS, NTLT, NTLbuf3, NTLbuf1, fail);
+    if (fail)
+      return;
+    S= convertNTLzz_pEX2CF (NTLS, x, M.mvar());
+    T= convertNTLzz_pEX2CF (NTLT, x, M.mvar());
+#else
     buf1= i.getItem();
     tryExtgcd (buf3, buf1, M, buf3, S, T, fail);
     if (fail)
       return;
+#endif
     CFListIterator k= factors;
     for (CFListIterator j= result; j.hasItem(); j++, k++)
     {
@@ -414,12 +447,14 @@ diophantineHensel (const CanonicalForm & F, const CFList& factors,
   CanonicalForm g;
   CanonicalForm modulus= p;
   int d= b.getk();
+  modpk b2;
   for (int i= 1; i < d; i++)
   {
     coeffE= div (e, modulus);
     setCharacteristic (p);
     coeffE= coeffE.mapinto();
     setCharacteristic (0);
+    b2= modpk (p, d - i);
     if (!coeffE.isZero())
     {
       CFListIterator k= result;
@@ -429,11 +464,12 @@ diophantineHensel (const CanonicalForm & F, const CFList& factors,
       for (; j.hasItem(); j++, k++, l++, ii++)
       {
         setCharacteristic (p);
-        g= mulNTL (coeffE, j.getItem());
+        g= modNTL (coeffE, bufFactors[ii]);
+        g= mulNTL (g, j.getItem());
         g= modNTL (g, bufFactors[ii]);
         setCharacteristic (0);
         k.getItem() += g.mapinto()*modulus;
-        e -= mulNTL (g.mapinto()*modulus, l.getItem(), b);
+        e -= mulNTL (g.mapinto(), b2 (l.getItem()), b2)*modulus;
         e= b(e);
       }
     }
@@ -532,25 +568,19 @@ diophantineHenselQa (const CanonicalForm & F, const CanonicalForm& G,
   setCharacteristic (p);
   Variable beta;
   Off (SW_RATIONAL);
-  if (mipoHasDen)
-  {
-    setReduce (alpha, false);
-    modMipo= modMipo.mapinto();
-    modMipo /= lc (modMipo);
-    beta= rootOf (modMipo);
-    setReduce (alpha, true);
-  }
+  setReduce (alpha, false);
+  modMipo= modMipo.mapinto();
+  modMipo /= lc (modMipo);
+  beta= rootOf (modMipo);
+  setReduce (alpha, true);
 
+  setReduce (alpha, false);
   for (k= 0; k < factors.length(); k++)
   {
-    if (!mipoHasDen)
-      bufFactors [k]= bufFactors[k].mapinto();
-    else
-    {
-      bufFactors [k]= bufFactors[k].mapinto();
-      bufFactors [k]= replacevar (bufFactors[k], alpha, beta);
-    }
+    bufFactors [k]= bufFactors[k].mapinto();
+    bufFactors [k]= replacevar (bufFactors[k], alpha, beta);
   }
+  setReduce (alpha, true);
   setCharacteristic(0);
 
   CFListIterator j= L;
@@ -588,25 +618,37 @@ diophantineHenselQa (const CanonicalForm & F, const CanonicalForm& G,
       i.getItem()= replacevar (i.getItem(), alpha, gamma);
   }
   setCharacteristic (p);
+  setReduce (alpha, false);
   recResult= mapinto (recResult);
-  if (mipoHasDen)
-  {
-    for (CFListIterator i= recResult; i.hasItem(); i++)
-      i.getItem()= replacevar (i.getItem(), alpha, beta);
-  }
+  setReduce (alpha, true);
+
+  for (CFListIterator i= recResult; i.hasItem(); i++)
+    i.getItem()= replacevar (i.getItem(), alpha, beta);
 
   setCharacteristic (0);
   CanonicalForm g;
   CanonicalForm modulus= p;
   int d= b.getk();
+  modpk b2;
   for (int i= 1; i < d; i++)
   {
     coeffE= div (e, modulus);
     setCharacteristic (p);
+    if (mipoHasDen)
+      setReduce (gamma, false);
+    else
+      setReduce (alpha, false);
     coeffE= coeffE.mapinto();
+    if (mipoHasDen)
+      setReduce (gamma, true);
+    else
+      setReduce (alpha, true);
     if (mipoHasDen)
       coeffE= replacevar (coeffE, gamma, beta);
+    else
+      coeffE= replacevar (coeffE, alpha, beta);
     setCharacteristic (0);
+    b2= modpk (p, d - i);
     if (!coeffE.isZero())
     {
       CFListIterator k= result;
@@ -616,19 +658,25 @@ diophantineHenselQa (const CanonicalForm & F, const CanonicalForm& G,
       for (; j.hasItem(); j++, k++, l++, ii++)
       {
         setCharacteristic (p);
-        g= mulNTL (coeffE, j.getItem());
+        g= modNTL (coeffE, bufFactors[ii]);
+        g= mulNTL (g, j.getItem());
         g= modNTL (g, bufFactors[ii]);
         setCharacteristic (0);
         if (mipoHasDen)
         {
+          setReduce (beta, false);
           k.getItem() += replacevar (g.mapinto()*modulus, beta, gamma);
-          e -= mulNTL (replacevar (g.mapinto(), beta, gamma)*modulus,
-                                   l.getItem(), b);
+          e -= mulNTL (replacevar (g.mapinto(), beta, gamma),
+                       b2 (l.getItem()), b2)*modulus;
+          setReduce (beta, true);
         }
         else
         {
-          k.getItem() += g.mapinto()*modulus;
-          e -= mulNTL (g.mapinto()*modulus, l.getItem(), b);
+          setReduce (beta, false);
+          k.getItem() += replacevar (g.mapinto()*modulus, beta, alpha);
+          e -= mulNTL (replacevar (g.mapinto(), beta, alpha),
+                       b2 (l.getItem()), b2)*modulus;
+          setReduce (beta, true);
         }
         e= b(e);
       }
@@ -641,6 +689,117 @@ diophantineHenselQa (const CanonicalForm & F, const CanonicalForm& G,
   return result;
 }
 
+CFList
+diophantineQa (const CanonicalForm& F, const CanonicalForm& G,
+               const CFList& factors, modpk& b, const Variable& alpha)
+{
+  bool fail= false;
+  CFList recResult;
+  CanonicalForm modMipo, mipo;
+  //here SW_RATIONAL is off
+  On (SW_RATIONAL);
+  mipo= getMipo (alpha);
+  bool mipoHasDen= false;
+  if (!bCommonDen (mipo).isOne())
+  {
+    mipo *= bCommonDen (mipo);
+    mipoHasDen= true;
+  }
+  Off (SW_RATIONAL);
+  int p= b.getp();
+  setCharacteristic (p);
+  setReduce (alpha, false);
+  while (1)
+  {
+    setCharacteristic (p);
+    modMipo= mapinto (mipo);
+    modMipo /= lc (modMipo);
+    tryDiophantine (recResult, mapinto (F), mapinto (factors), modMipo, fail);
+    if (fail)
+    {
+      int i= 0;
+      while (cf_getBigPrime (i) < p)
+        i++;
+      findGoodPrime (F, i);
+      findGoodPrime (G, i);
+      p=cf_getBigPrime(i);
+      b = coeffBound( G, p, mipo );
+      modpk bb= coeffBound (F, p, mipo );
+      if (bb.getk() > b.getk() ) b=bb;
+      fail= false;
+    }
+    else
+      break;
+  }
+  setReduce (alpha, true);
+  setCharacteristic (0);
+
+  Variable gamma= alpha;
+  CanonicalForm den;
+  if (mipoHasDen)
+  {
+    On (SW_RATIONAL);
+    modMipo= getMipo (alpha);
+    den= bCommonDen (modMipo);
+    modMipo *= den;
+    Off (SW_RATIONAL);
+    setReduce (alpha, false);
+    gamma= rootOf (b (modMipo*b.inverse (den)));
+    setReduce (alpha, true);
+  }
+
+  Variable x= Variable (1);
+  CanonicalForm buf1, buf2, buf3, S;
+  CFList bufFactors= factors;
+  CFListIterator i= bufFactors;
+  if (mipoHasDen)
+  {
+    for (; i.hasItem(); i++)
+      i.getItem()= replacevar (i.getItem(), alpha, gamma);
+  }
+  i= bufFactors;
+  CFList result;
+  if (i.hasItem())
+    i++;
+  buf1= 0;
+  CanonicalForm Freplaced;
+  if (mipoHasDen)
+  {
+    Freplaced= replacevar (F, alpha, gamma);
+    buf2= divNTL (Freplaced, replacevar (i.getItem(), alpha, gamma), b);
+  }
+  else
+    buf2= divNTL (F, i.getItem(), b);
+  ZZ_p::init (convertFacCF2NTLZZ (b.getpk()));
+  ZZ_pX NTLmipo= to_ZZ_pX (convertFacCF2NTLZZX (getMipo (gamma)));
+  ZZ_pE::init (NTLmipo);
+  ZZ_pEX NTLS, NTLT, NTLbuf3;
+  ZZ_pEX NTLbuf1= convertFacCF2NTLZZ_pEX (buf1, NTLmipo);
+  ZZ_pEX NTLbuf2= convertFacCF2NTLZZ_pEX (buf2, NTLmipo);
+  XGCD (NTLbuf3, NTLS, NTLT, NTLbuf1, NTLbuf2);
+  result.append (b (convertNTLZZ_pEX2CF (NTLS, x, gamma)));
+  result.append (b (convertNTLZZ_pEX2CF (NTLT, x, gamma)));
+  if (i.hasItem())
+    i++;
+  for (; i.hasItem(); i++)
+  {
+    if (mipoHasDen)
+      buf1= divNTL (Freplaced, i.getItem(), b);
+    else
+      buf1= divNTL (F, i.getItem(), b);
+    XGCD (NTLbuf3, NTLS, NTLT, NTLbuf3, convertFacCF2NTLZZ_pEX (buf1, NTLmipo));
+    CFListIterator k= bufFactors;
+    S= convertNTLZZ_pEX2CF (NTLS, x, gamma);
+    for (CFListIterator j= result; j.hasItem(); j++, k++)
+    {
+      j.getItem()= mulNTL (j.getItem(), S, b);
+      j.getItem()= modNTL (j.getItem(), k.getItem(), b);
+    }
+    result.append (b (convertNTLZZ_pEX2CF (NTLT, x, gamma)));
+  }
+  return result;
+}
+
 CFList
 diophantine (const CanonicalForm& F, const CanonicalForm& G,
              const CFList& factors, modpk& b)
@@ -655,7 +814,7 @@ diophantine (const CanonicalForm& F, const CanonicalForm& G,
     {
       if (b.getp() != 0)
       {
-        CFList result= diophantineHenselQa (F, G, factors, b, v);
+        CFList result= diophantineQa (F, G, factors, b, v);
         return result;
       }
       CFList result= modularDiophant (F, factors, getMipo (v));
