@@ -402,7 +402,14 @@ mulNTL (const CanonicalForm& F, const CanonicalForm& G, const modpk& b)
       if (b.getp() != 0)
       {
         ZZ_p::init (convertFacCF2NTLZZ (b.getpk()));
-        ZZ_pX NTLmipo= to_ZZ_pX (convertFacCF2NTLZZX (getMipo (alpha)));
+        CanonicalForm mipo= getMipo (alpha);
+        bool is_rat= isOn (SW_RATIONAL);
+        if (!is_rat)
+          On (SW_RATIONAL);
+        mipo *=bCommonDen (mipo);
+        if (!is_rat)
+          Off (SW_RATIONAL);
+        ZZ_pX NTLmipo= to_ZZ_pX (convertFacCF2NTLZZX (mipo));
         ZZ_pE::init (NTLmipo);
         ZZ_pEX NTLg= convertFacCF2NTLZZ_pEX (G, NTLmipo);
         ZZ_pEX NTLf= convertFacCF2NTLZZ_pEX (F, NTLmipo);
@@ -494,7 +501,11 @@ mulNTL (const CanonicalForm& F, const CanonicalForm& G, const modpk& b)
     return F*G;
   ASSERT (F.isUnivariate() && G.isUnivariate(), "expected univariate polys");
   ASSERT (F.level() == G.level(), "expected polys of same level");
-  zz_p::init (getCharacteristic());
+  if (fac_NTL_char != getCharacteristic())
+  {
+    fac_NTL_char= getCharacteristic();
+    zz_p::init (getCharacteristic());
+  }
   Variable alpha;
   CanonicalForm result;
   if (hasFirstAlgVar (F, alpha) || hasFirstAlgVar (G, alpha))
@@ -610,7 +621,11 @@ modNTL (const CanonicalForm& F, const CanonicalForm& G, const modpk& b)
 
   ASSERT (F.isUnivariate() && G.isUnivariate(), "expected univariate polys");
   ASSERT (F.level() == G.level(), "expected polys of same level");
-  zz_p::init (getCharacteristic());
+  if (fac_NTL_char != getCharacteristic())
+  {
+    fac_NTL_char= getCharacteristic();
+    zz_p::init (getCharacteristic());
+  }
   Variable alpha;
   CanonicalForm result;
   if (hasFirstAlgVar (F, alpha) || hasFirstAlgVar (G, alpha))
@@ -755,7 +770,11 @@ divNTL (const CanonicalForm& F, const CanonicalForm& G, const modpk& b)
 
   ASSERT (F.isUnivariate() && G.isUnivariate(), "expected univariate polys");
   ASSERT (F.level() == G.level(), "expected polys of same level");
-  zz_p::init (getCharacteristic());
+  if (fac_NTL_char != getCharacteristic())
+  {
+    fac_NTL_char= getCharacteristic();
+    zz_p::init (getCharacteristic());
+  }
   Variable alpha;
   CanonicalForm result;
   if (hasFirstAlgVar (F, alpha) || hasFirstAlgVar (G, alpha))
@@ -2065,7 +2084,11 @@ mulMod2NTLFq (const CanonicalForm& F, const CanonicalForm& G, const
     int degBy= degree (B, 2);
     int d1= degAx + degBx + 1;
     int d2= tmax (degAy, degBy);
-    zz_p::init (getCharacteristic());
+    if (fac_NTL_char != getCharacteristic())
+    {
+      fac_NTL_char= getCharacteristic();
+      zz_p::init (getCharacteristic());
+    }
     zz_pX NTLMipo= convertFacCF2NTLzzpX (getMipo (alpha));
     zz_pE::init (NTLMipo);
 
@@ -2193,6 +2216,10 @@ CanonicalForm mulMod (const CanonicalForm& A, const CanonicalForm& B,
   CanonicalForm G= mod (B, M);
   if (F.inCoeffDomain() || G.inCoeffDomain())
     return F*G;
+
+  if (size (F) / MOD.length() < 100 || size (G) / MOD.length() < 100)
+    return mod (F*G, MOD);
+
   Variable y= M.mvar();
   int degF= degree (F, y);
   int degG= degree (G, y);
@@ -2720,13 +2747,6 @@ void divrem2 (const CanonicalForm& F, const CanonicalForm& G, CanonicalForm& Q,
     divrem (A, B, Q, R);
     return;
   }
-  if (!(B.level() == 1 && B.isUnivariate()) &&
-      (A.level() == 1 && A.isUnivariate()))
-  {
-    Q= 0;
-    R= A;
-    return;
-  }
 
   Variable x= Variable (1);
   int degB= degree (B, x);
@@ -2821,7 +2841,11 @@ uniFdivides (const CanonicalForm& A, const CanonicalForm& B)
   }
   if (p > 0)
   {
-    zz_p::init (p);
+    if (fac_NTL_char != p)
+    {
+      fac_NTL_char= p;
+      zz_p::init (p);
+    }
     Variable alpha;
     if (hasFirstAlgVar (A, alpha) || hasFirstAlgVar (B, alpha))
     {
