@@ -35,9 +35,6 @@
 #include <kernel/p_Mult_q.h>
 #include <kernel/pInline1.h>
 
-// dirty tricks:
-#include <kernel/p_MemAdd.h>
-
 #include <kernel/gring.h>
 #include <kernel/sca.h>
 #include <summator.h>
@@ -200,15 +197,17 @@ poly p_Lcm(const poly a, const poly b, const ring r)
 
 ///////////////////////////////////////////////////////////////////////////////
 poly nc_p_Minus_mm_Mult_qq(poly p, const poly m, const poly q, int &lp,
-                                    const int, const poly, const ring r)
+                                    const poly, const ring r)
 {
   poly mc  = p_Neg( p_Copy(m, r), r );
   poly mmc = nc_mm_Mult_pp( mc, q, r );
   p_Delete(&mc, r);
 
+  int org_p=pLength(p);
+  int org_q=pLength(q);
   p = p_Add_q(p, mmc, r);
 
-  lp = pLength(p); // ring independent!
+  lp = pLength(p)-org_p-org_q; // ring independent!
 
   return(p);
 }
@@ -438,7 +437,7 @@ poly gnc_p_Mult_mm_Common(poly p, const poly m, int side, const ring r)
 }
 
 /* poly functions defined in p_Procs : */
-poly gnc_pp_Mult_mm(const poly p, const poly m, const ring r, poly &last)
+poly gnc_pp_Mult_mm(const poly p, const poly m, const ring r)
 {
   return( gnc_p_Mult_mm_Common(p_Copy(p,r), m, 1, r) );
 }
@@ -458,8 +457,6 @@ poly gnc_mm_Mult_pp(const poly m, const poly p, const ring r)
   return( gnc_p_Mult_mm_Common(p_Copy(p,r), m, 0, r) );
 }
 
-
-
 poly gnc_mm_Mult_nn(int *F0, int *G0, const ring r)
 /* destroys nothing, no coeffs and exps */
 {
@@ -467,7 +464,6 @@ poly gnc_mm_Mult_nn(int *F0, int *G0, const ring r)
   int i,j;
   int iF,jG,iG;
   int rN=r->N;
-  int ExpSize=(((rN+1)*sizeof(int)+sizeof(long)-1)/sizeof(long))*sizeof(long);
 
   int *F=(int *)omAlloc0((rN+1)*sizeof(int));
   int *G=(int *)omAlloc0((rN+1)*sizeof(int));
@@ -476,7 +472,7 @@ poly gnc_mm_Mult_nn(int *F0, int *G0, const ring r)
   // pExpVectorCopy(F,F0);
   memcpy(G, G0,(rN+1)*sizeof(int));
   //  pExpVectorCopy(G,G0);
-  F[0]=0; /* important for p_MemAdd */
+  F[0]=0;
   G[0]=0;
 
   iF=rN;
@@ -500,10 +496,9 @@ poly gnc_mm_Mult_nn(int *F0, int *G0, const ring r)
   if (iF<=jG)
     /* i.e. no mixed exp_num , MERGE case */
   {
-    p_MemAdd_LengthGeneral(F, G, ExpSize/sizeof(long));
+    { for(int ii=rN;ii>0;ii--) F[ii]+=G[ii]; }
     p_SetExpV(out,F,r);
     p_Setm(out,r);
-    //    omFreeSize((ADDRESS)F,ExpSize);
     freeT(F,rN);
     freeT(G,rN);
     return(out);
@@ -562,11 +557,10 @@ poly gnc_mm_Mult_nn(int *F0, int *G0, const ring r)
       }
       cff=totcff;
     }
-    p_MemAdd_LengthGeneral(F, G, ExpSize/sizeof(long));
+    { for(int ii=rN;ii>0;ii--) F[ii]+=G[ii]; }
     p_SetExpV(out,F,r);
     p_Setm(out,r);
     p_SetCoeff(out,cff,r);
-    //    p_MemAdd_NegWeightAdjust(p, r); ??? do we need this?
     freeT(F,rN);
     freeT(G,rN);
     return(out);
@@ -3354,10 +3348,7 @@ void gnc_p_ProcsSet(ring rGR, p_Procs_s* p_Procs)
   // "commutative"
   p_Procs->p_Mult_mm  = rGR->p_Procs->p_Mult_mm  = gnc_p_Mult_mm;
   p_Procs->pp_Mult_mm = rGR->p_Procs->pp_Mult_mm = gnc_pp_Mult_mm;
-  p_Procs->p_Minus_mm_Mult_qq = rGR->p_Procs->p_Minus_mm_Mult_qq = NULL;
-  // gnc_p_Minus_mm_Mult_qq_ign; // should not be used!!!???
-
-
+  p_Procs->p_Minus_mm_Mult_qq = rGR->p_Procs->p_Minus_mm_Mult_qq = nc_p_Minus_mm_Mult_qq;
 
   // non-commutaitve multiplication by monomial from the left
   rGR->GetNC()->p_Procs.mm_Mult_p   = gnc_mm_Mult_p;
