@@ -132,7 +132,9 @@ void id_Delete (ideal * h, ring r)
   {
     do
     {
-      p_Delete(&((*h)->m[--j]), r);
+      j--;
+      poly pp=(*h)->m[j];
+      if (pp!=NULL) p_Delete(&pp, r);
     }
     while (j>0);
     omFreeSize((ADDRESS)((*h)->m),sizeof(poly)*elems);
@@ -894,81 +896,6 @@ int pLowVar (poly p)
   return k;
 }
 
-/*3
-*multiplies p with t (!cas) or  (t-1)
-*the index of t is:1, so we have to shift all variables
-*p is NOT in the actual ring, it has no t
-*/
-static poly pMultWithT (poly p,BOOLEAN cas)
-{
-  /*qp is the working pointer in p*/
-  /*result is the result, qresult is the working pointer*/
-  /*pp is p in the actual ring(shifted), qpp the working pointer*/
-  poly result,qp,pp;
-  poly qresult=NULL;
-  poly qpp=NULL;
-  int  i,j,lex;
-  number n;
-
-  pp = NULL;
-  result = NULL;
-  qp = p;
-  while (qp != NULL)
-  {
-    i = 0;
-    if (result == NULL)
-    {/*first monomial*/
-      result = pInit();
-      qresult = result;
-    }
-    else
-    {
-      qresult->next = pInit();
-      pIter(qresult);
-    }
-    for (j=pVariables-1; j>0; j--)
-    {
-      lex = pGetExp(qp,j);
-      pSetExp(qresult,j+1,lex);/*copy all variables*/
-    }
-    lex = pGetComp(qp);
-    pSetComp(qresult,lex);
-    n=nCopy(pGetCoeff(qp));
-    pSetCoeff0(qresult,n);
-    qresult->next = NULL;
-    pSetm(qresult);
-    /*qresult is now qp brought into the actual ring*/
-    if (cas)
-    { /*case: mult with t-1*/
-      pSetExp(qresult,1,0);
-      pSetm(qresult);
-      if (pp == NULL)
-      { /*first monomial*/
-        pp = pCopy(qresult);
-        qpp = pp;
-      }
-      else
-      {
-        qpp->next = pCopy(qresult);
-        pIter(qpp);
-      }
-      pGetCoeff(qpp)=nNeg(pGetCoeff(qpp));
-      /*now qpp contains -1*qp*/
-    }
-    pSetExp(qresult,1,1);/*this is mult. by t*/
-    pSetm(qresult);
-    pIter(qp);
-  }
-  /*
-  *now p is processed:
-  *result contains t*p
-  * if cas: pp contains -1*p (in the new ring)
-  */
-  if (cas)  qresult->next = pp;
-  /*  else      qresult->next = NULL;*/
-  return result;
-}
-
 /*2
 * verschiebt die Indizees der Modulerzeugenden um i
 */
