@@ -536,12 +536,11 @@ KINLINE void sLObject::PrepareRed(BOOLEAN use_bucket)
       pNext(tp) = NULL;
       if (p != NULL) pNext(p) = NULL;
       pLength = 0;
-      last = NULL;
     }
   }
 }
 
-KINLINE void sLObject::SetLmTail(poly lm, poly p_tail, int p_Length, int use_bucket, ring _tailRing, poly _last)
+KINLINE void sLObject::SetLmTail(poly lm, poly p_tail, int p_Length, int use_bucket, ring _tailRing)
 {
 
   Set(lm, _tailRing);
@@ -551,13 +550,11 @@ KINLINE void sLObject::SetLmTail(poly lm, poly p_tail, int p_Length, int use_buc
     kBucketInit(bucket, p_tail, p_Length);
     pNext(lm) = NULL;
     pLength = 0;
-    last = NULL;
   }
   else
   {
     pNext(lm) = p_tail;
     pLength = p_Length + 1;
-    last = _last;
   }
 
 }
@@ -590,7 +587,7 @@ KINLINE void sLObject::Tail_Minus_mm_Mult_qq(poly m, poly q, int lq,
     int shorter;
     pNext(_p) = tailRing->p_Procs->p_Minus_mm_Mult_qq(pNext(_p), m, q,
                                                       shorter,spNoether,
-                                                      tailRing, last);
+                                                      tailRing);
     pLength += lq - shorter;
   }
 }
@@ -708,7 +705,6 @@ sLObject::ShallowCopyDelete(ring new_tailRing,
   sTObject::ShallowCopyDelete(new_tailRing,
                               new_tailRing->PolyBin,p_shallow_copy_delete,
                               FALSE);
-  last = NULL;
 }
 
 KINLINE void sLObject::SetShortExpVector()
@@ -737,12 +733,10 @@ KINLINE void sLObject::Copy()
     if (p != NULL) pNext(p) = NULL;
   }
   TObject::Copy();
-  last = NULL;
 }
 
 KINLINE poly sLObject::CopyGetP()
 {
-  last = NULL;
   if (bucket != NULL)
   {
     int i = kBucketCanonicalize(bucket);
@@ -779,18 +773,16 @@ KINLINE long sLObject::pLDeg(BOOLEAN deg_last)
 {
   if (! deg_last || bucket != NULL) return sLObject::pLDeg();
 
-  if (last == NULL || pLength == 0)
-    last = pLast(GetLmTailRing(), pLength);
-#ifdef HAVE_ASSUME
   long ldeg;
   ldeg = tailRing->pLDeg(GetLmTailRing(), &length, tailRing);
+#ifdef HAVE_ASSUME
+  if ( pLength == 0)
+    p_Last(GetLmTailRing(), pLength, tailRing);
   assume ( pLength == length || rIsSyzIndexRing(currRing));
-  assume (ldeg == tailRing->pFDeg(last,tailRing));
-  return ldeg;
 #else
-  length = pLength;
-  return tailRing->pFDeg(last, tailRing);
+  pLength=length;
 #endif
+  return ldeg;
 }
 
 KINLINE long sLObject::SetDegStuffReturnLDeg()
