@@ -714,7 +714,6 @@ int redHoney (LObject* h, kStrategy strat)
         if(at <= strat->Ll)
           /*- h will not become the next element to reduce -*/
         {
-          h->last=NULL;
           enterL(&strat->L,&strat->Ll,&strat->Lmax,*h,at);
 #ifdef KDEBUG
           if (TEST_OPT_DEBUG) Print(" ecart too big: -> L%d\n",at);
@@ -779,7 +778,6 @@ int redHoney (LObject* h, kStrategy strat)
       if (at <= strat->Ll)
       {
         int dummy=strat->sl;
-        h->last=NULL;
         if (kFindDivisibleByInS(strat, &dummy, h) < 0)
           return 1;
         enterL(&strat->L,&strat->Ll,&strat->Lmax,*h,at);
@@ -801,7 +799,6 @@ int redHoney (LObject* h, kStrategy strat)
           //Print("OVERFLOW in redHoney d=%ld, max=%ld\n",d,strat->tailRing->bitmask);
           h->GetP();
           at = strat->posInL(strat->L,strat->Ll,h,strat);
-          h->last=NULL;
           enterL(&strat->L,&strat->Ll,&strat->Lmax,*h,at);
           h->Clear();
           return -1;
@@ -1349,10 +1346,8 @@ poly kNF2 (ideal F,ideal Q,poly q,kStrategy strat, int lazyReduce)
     else
     #endif
     {
-      BITSET save=test;
       test &= ~Sy_bit(OPT_INTSTRATEGY);
       p = redtailBba(p,max_ind,strat,(lazyReduce & KSTD_NF_NONORM)==0);
-      test=save;
     }
   }
   /*- release temp data------------------------------- -*/
@@ -1405,7 +1400,6 @@ ideal kNF2 (ideal F,ideal Q,ideal q,kStrategy strat, int lazyReduce)
   /*Shdl=*/initS(F,Q,strat);
   /*- compute------------------------------------------------------- -*/
   res=idInit(IDELEMS(q),si_max(q->rank,F->rank));
-  BITSET save=test;
   test &= ~Sy_bit(OPT_INTSTRATEGY);
   for (i=IDELEMS(q)-1; i>=0; i--)
   {
@@ -1433,7 +1427,6 @@ ideal kNF2 (ideal F,ideal Q,ideal q,kStrategy strat, int lazyReduce)
     //  res->m[i]=NULL;
   }
   /*- release temp data------------------------------- -*/
-  test=save;
   omfree(strat->sevS);
   omfree(strat->ecartS);
   omfree(strat->T);
