@@ -296,7 +296,9 @@ PINLINE2 long p_GetExpDiff(poly p1, poly p2, int i, ring r)
 PINLINE2 poly p_New(ring r, omBin bin)
 {
   p_CheckRing2(r);
+  #ifndef X_OMALLOC
   pAssume2(bin != NULL && r->PolyBin->sizeW == bin->sizeW);
+  #endif
   poly p;
   omTypeAllocBin(poly, p, bin);
   p_SetRingOfLm(p, r);
@@ -478,7 +480,9 @@ PINLINE2 void p_Delete(poly *p,  const ring lmRing, const ring tailRing)
 PINLINE2 poly p_ShallowCopyDelete(poly p, const ring r, omBin bin)
 {
   p_LmCheckPolyRing2(p, r);
+  #ifndef X_OMALLOC
   pAssume2(r->PolyBin->sizeW == bin->sizeW);
+  #endif
   return r->p_Procs->p_ShallowCopyDelete(p, r, bin);
 }
 
@@ -538,8 +542,7 @@ PINLINE2 poly pp_Mult_mm(poly p, poly m, const ring r)
     return pp_Mult_nn(p, pGetCoeff(m), r);
   else
   {
-    poly last;
-    return r->p_Procs->pp_Mult_mm(p, m, r, last);
+    return r->p_Procs->pp_Mult_mm(p, m, r);
   }
 }
 
@@ -555,32 +558,17 @@ PINLINE2 poly p_Mult_mm(poly p, poly m, const ring r)
 // return p - m*Copy(q), destroys p; const: p,m
 PINLINE2 poly p_Minus_mm_Mult_qq(poly p, poly m, poly q, const ring r)
 {
-#ifdef HAVE_PLURAL
-  if (rIsPluralRing(r))
-  {
-    int lp, lq;
-    poly spNoether;
-    return nc_p_Minus_mm_Mult_qq(p, m, q, lp, lq, spNoether, r);
-  }
-#endif
-
   int shorter;
-  poly last;
 
-  return r->p_Procs->p_Minus_mm_Mult_qq(p, m, q, shorter, NULL, r, last); // !!!
+  return r->p_Procs->p_Minus_mm_Mult_qq(p, m, q, shorter, NULL, r);
 }
 
 PINLINE2 poly p_Minus_mm_Mult_qq(poly p, poly m, poly q, int &lp, int lq,
                                  poly spNoether, const ring r)
 {
-#ifdef HAVE_PLURAL
-  if (rIsPluralRing(r))
-     return nc_p_Minus_mm_Mult_qq(p, m, q, lp, lq, spNoether, r);
-#endif
-
   int shorter;
-  poly last,res;
-  res = r->p_Procs->p_Minus_mm_Mult_qq(p, m, q, shorter, spNoether, r, last);
+  poly res;
+  res = r->p_Procs->p_Minus_mm_Mult_qq(p, m, q, shorter, spNoether, r);
   lp = (lp + lq) - shorter;
   return res;
 }
@@ -657,7 +645,6 @@ PINLINE2 poly p_Mult_q(poly p, poly q, const ring r)
 // returns p*q, does neither destroy p nor q
 PINLINE2 poly pp_Mult_qq(poly p, poly q, const ring r)
 {
-  poly last;
   if (p == NULL || q == NULL) return NULL;
 
   if (pNext(p) == NULL)
@@ -666,12 +653,12 @@ PINLINE2 poly pp_Mult_qq(poly p, poly q, const ring r)
     if (rIsPluralRing(r))
       return nc_mm_Mult_pp(p, q, r);
 #endif
-    return r->p_Procs->pp_Mult_mm(q, p, r, last);
+    return r->p_Procs->pp_Mult_mm(q, p, r);
   }
 
   if (pNext(q) == NULL)
   {
-    return r->p_Procs->pp_Mult_mm(p, q, r, last);
+    return r->p_Procs->pp_Mult_mm(p, q, r);
   }
 
   poly qq = q;
@@ -701,13 +688,13 @@ PINLINE2 poly p_Plus_mm_Mult_qq(poly p, poly m, poly q, int &lp, int lq,
 #endif
 
 // this should be implemented more efficiently
-  poly res, last;
+  poly res;
   int shorter;
   number n_old = pGetCoeff(m);
   number n_neg = n_Copy(n_old, r);
   n_neg = n_Neg(n_neg, r);
   pSetCoeff0(m, n_neg);
-  res = r->p_Procs->p_Minus_mm_Mult_qq(p, m, q, shorter, NULL, r, last);
+  res = r->p_Procs->p_Minus_mm_Mult_qq(p, m, q, shorter, NULL, r);
   lp = (lp + lq) - shorter;
   pSetCoeff0(m, n_old);
   n_Delete(&n_neg, r);
