@@ -541,14 +541,14 @@ public:
   SparseRow<number_type>(int n)
   {
     len=n;
-    idx_array=(int*) omalloc(n*sizeof(int));
-    coef_array=(number_type*) omalloc(n*sizeof(number_type));
+    idx_array=(int*) omAlloc(n*sizeof(int));
+    coef_array=(number_type*) omAlloc(n*sizeof(number_type));
   }
   SparseRow<number_type>(int n, const number_type* source)
   {
     len=n;
     idx_array=NULL;
-    coef_array=(number_type*) omalloc(n*sizeof(number_type));
+    coef_array=(number_type*) omAlloc(n*sizeof(number_type));
     memcpy(coef_array,source,n*sizeof(number_type));
   }
   ~SparseRow<number_type>()
@@ -1395,7 +1395,7 @@ template<class number_type> SparseRow<number_type> * noro_red_to_non_poly_t(poly
     return NULL;
   }
 
-  MonRedResNP<number_type>* mon=(MonRedResNP<number_type>*) omalloc(len*sizeof(MonRedResNP<number_type>));
+  MonRedResNP<number_type>* mon=(MonRedResNP<number_type>*) omAlloc(len*sizeof(MonRedResNP<number_type>));
   int i=0;
   double max_density=0.0;
   while(p!=NULL)
@@ -1649,9 +1649,10 @@ public:
     this->nrows=p.nrows;
     lastReducibleIndices=(int*) omalloc(nrows*sizeof(int));
     nonZeroUntil=0;
-    while(nonZeroUntil<nrows){
-      if (startIndices[nonZeroUntil]<ncols){
-
+    while(nonZeroUntil<nrows)
+    {
+      if (startIndices[nonZeroUntil]<ncols)
+      {
         nonZeroUntil++;
       }
       else break;
@@ -1767,7 +1768,7 @@ template <class number_type> void noro_step(poly*p,int &pn,slimgb_alg* c){
 
   NoroCache<number_type> cache;
 
-  SparseRow<number_type> ** srows=(SparseRow<number_type>**) omalloc(pn*sizeof(SparseRow<number_type>*));
+  SparseRow<number_type> ** srows=(SparseRow<number_type>**) omAlloc(pn*sizeof(SparseRow<number_type>*));
   int non_zeros=0;
   for(j=0;j<pn;j++){
 
@@ -1786,13 +1787,15 @@ template <class number_type> void noro_step(poly*p,int &pn,slimgb_alg* c){
   //Print("historic irred Mon%d\n",cache.nIrreducibleMonomials);
   int n=irr_nodes.size();//cache.countIrreducibleMonomials();
   cache.nIrreducibleMonomials=n;
-  if (TEST_OPT_PROT){
+  if (TEST_OPT_PROT)
+  {
     Print("Irred Mon:%d\n",n);
     Print("red Mon:%d\n",cache.nReducibleMonomials);
   }
   TermNoroDataNode<number_type>* term_nodes=(TermNoroDataNode<number_type>*) omalloc(n*sizeof(TermNoroDataNode<number_type>));
 
-  for(j=0;j<n;j++){
+  for(j=0;j<n;j++)
+  {
     assume(irr_nodes[j]!=NULL);
     assume(irr_nodes[j]->value_len==NoroCache<number_type>::backLinkCode);
     term_nodes[j].t=irr_nodes[j]->value_poly;
@@ -1800,11 +1803,10 @@ template <class number_type> void noro_step(poly*p,int &pn,slimgb_alg* c){
     term_nodes[j].node=irr_nodes[j];
   }
 
-
   qsort(term_nodes,n,sizeof(TermNoroDataNode<number_type>),term_nodes_sort_crit<number_type>);
   poly* terms=(poly*) omalloc(n*sizeof(poly));
 
-  int* old_to_new_indices=(int*) omalloc(cache.nIrreducibleMonomials*sizeof(int));
+  int* old_to_new_indices=(int*) omalloc(n /*cache.nIrreducibleMonomials*/ *sizeof(int));
   for(j=0;j<n;j++)
   {
     old_to_new_indices[term_nodes[j].node->term_index]=j;
@@ -1815,8 +1817,8 @@ template <class number_type> void noro_step(poly*p,int &pn,slimgb_alg* c){
   //if (TEST_OPT_PROT)
   //  Print("Evaluate Rows \n");
   pn=non_zeros;
-  number_type* number_array=(number_type*) omalloc(n*pn*sizeof(number_type));
-  memset(number_array,0,sizeof(number_type)*n*pn);
+  number_type* number_array=(number_type*) omalloc0(n*pn*sizeof(number_type));
+  //memset(number_array,0,sizeof(number_type)*n*pn);
 
   for(j=0;j<pn;j++)
   {
